// DISCLAIMER OF LIABILITY
// 
// This text/file contains proprietary, confidential
// information of Xilinx, Inc., is distributed under license
// from Xilinx, Inc., and may be used, copied and/or
// disclosed only pursuant to the terms of a valid license
// agreement with Xilinx, Inc. Xilinx hereby grants you a 
// license to use this text/file solely for design, simulation, 
// implementation and creation of design files limited 
// to Xilinx devices or technologies. Use with non-Xilinx 
// devices or technologies is expressly prohibited and 
// immediately terminates your license unless covered by
// a separate agreement.
//
// Xilinx is providing this design, code, or information 
// "as-is" solely for use in developing programs and 
// solutions for Xilinx devices, with no obligation on the 
// part of Xilinx to provide support. By providing this design, 
// code, or information as one possible implementation of 
// this feature, application or standard, Xilinx is making no 
// representation that this implementation is free from any 
// claims of infringement. You are responsible for 
// obtaining any rights you may require for your implementation. 
// Xilinx expressly disclaims any warranty whatsoever with 
// respect to the adequacy of the implementation, including 
// but not limited to any warranties or representations that this
// implementation is free from claims of infringement, implied 
// warranties of merchantability or fitness for a particular 
// purpose.
//
// Xilinx products are not intended for use in life support
// appliances, devices, or systems. Use in such applications is
// expressly prohibited.
//
// Any modifications that are made to the Source Code are 
// done at the users sole risk and will be unsupported.
//
//
// Copyright (c) 2007, 2008 Xilinx, Inc. All rights reserved.
//
// This copyright and support notice must be retained as part 
// of this text at all times. 
//
//******************************************************************************/
// Filename            : display_controller.c
// $Revision:: 4547   $: Revision of last commit
// $Date:: 2008-11-12#$: Date of last commit
// Description         : Implementation of display_controller Driver.
//******************************************************************************/

/***************************** Include Files *******************************/

#include "display_controller.h"

/************************** Function Definitions ***************************/


static Xuint16 ResolutionBuffer[DISPLAY_CONTROLLER_NUM_RESOLUTIONS][DISPLAY_CONTROLLER_NUM_CONFIG_PARAMS] = {
// VGA P60
	16,	//h front porch
	96,	//h sync width
	48,	//h back porch
	0,		//sync polarity
	10,	//v front porch
	2,		//v sync width
	33,	//v back porch
	640,	//line width
	480,	//frame height
	2560,	//minimum bytes per line based on 128 byte burst size
	0,		//interlaced format
// 480P60
	16,	//h front porch
	62,	//h sync width
	60,	//h back porch
	0,		//sync polarity
	9,		//v front porch
	6,		//v sync width
	30,	//v back porch
	720,	//line width
	480,	//frame height
	2944,	//minimum bytes per line based on 128 byte burst size
	0,		//interlaced format
// 576P50
	12,	//h front porch
	64,	//h sync width
	68,	//h back porch
	0,		//sync polarity
	5,		//v front porch
	5,		//v sync width
	39,	//v back porch
	720,	//line width
	576,	//frame height
	2944,	//minimum bytes per line based on 128 byte burst size
	0,		//interlaced format
// SVGA P60
	40,	//h front porch
	128,	//h sync width
	88,	//h back porch
	1,		//sync polarity
	1,		//v front porch
	4,		//v sync width
	23,	//v back porch
	800,	//line width
	600,	//frame height
	3200,	//minimum bytes per line based on 128 byte burst size
	0,		//interlaced format
// XGA P70
	24,		//h front porch
	136,	//h sync width
	144,	//h back porch
	1,		//sync polarity
	3,		//v front porch
	6,		//v sync width
	29,		//v back porch
	1024,	//line width
	768,	//frame height
	4096,	//minimum bytes per line based on 128 byte burst size
	0,		//interlaced format
// 720P50
	440,	//h front porch
	40,		//h sync width
	220,	//h back porch
	1,		//sync polarity
	5, 		//v front porch
	6,		//v sync width
	20,		//v back porch
	1280,	//line width
	720,	//frame height
	5120,	//minimum bytes per line based on 128 byte burst size
	0,		//interlaced format
// 720P60
	110,	//h front porch
	40,		//h sync width
	220,	//h back porch
	1,		//sync polarity
	5,		//v front porch
	5,		//v sync width
	20,		//v back porch
	1280,	//line width
	720,	//frame height
	5120,	//minimum bytes per line based on 128 byte burst size
	0,		//interlaced format
// 1080P25
	528,	//h front porch
	44,		//h sync width
	148,	//h back porch
	1,		//sync polarity
	4,		//v front porch
	5,		//v sync width
	36,		//v back porch
	1920,	//line width
	1080,	//frame height
	7680,	//minimum bytes per line based on 128 byte burst size
	0,		//interlaced format
// 1080P30
	88,		//h front porch
	44,		//h sync width
	148,	//h back porch
	1,		//sync polarity
	4,		//v front porch
	5,		//v sync width
	36,		//v back porch
	1920,	//line width
	1080,	//frame height
	7680,	//minimum bytes per line based on 128 byte burst size
	0,		//interlaced format
// 1080I50
	528,	//h front porch
	44,		//h sync width
	148,	//h back porch
	1,		//sync polarity
	2,		//v front porch
	5,		//v sync width
	15,		//v back porch
	1920,	//line width
	540,	//frame height
	15360,//minimum bytes per line based on 128 byte burst size
	1,		//interlaced format
// 1080I60
	88,		//h front porch
	44,		//h sync width
	148,	//h back porch
	1,		//sync polarity
	2,		//v front porch
	5,		//v sync width
	15,		//v back porch
	1920,	//line width
	540,	//frame height
	15360,//minimum bytes per line based on 128 byte burst size
	1,		//interlaced format
// NTSC
	28,		//38,	//h front porch
	124,	//h sync width
	124,	//114,	//h back porch
	0,		//sync polarity
	5,		//v front porch
	3,		//v sync width
	14,		//v back porch
	1440,	//line width
	240,	//frame height
	3072,	//minimum bytes per line based on 128 byte burst size
	1,		//interlaced format
// PAL
	20,		//h front porch
	126,	//h sync width
	142,	//h back porch
	0,		//sync polarity
	2,		//v front porch
	3,		//v sync width
	19,		//v back porch
	1440,	//line width
	288,	//frame height
	3072,	//minimum bytes per line based on 128 byte burst size
	1,		//interlaced format
// UTIA_ANA
	64,		//h front porch
	136,	//h sync width
	200,	//h back porch
	0,		//sync polarity
	1,		//v front porch
	3,		//v sync width
	24,		//v back porch
	1280,	//line width
	792,	//frame height
	5120,	//minimum bytes per line based on 128 byte burst size
	0,		//interlaced format
// UTIA_DIG
	64,		//h front porch
	136,	//h sync width
	200,	//h back porch
	0,		//sync polarity
	1,		//v front porch
	3,		//v sync width
	24,		//v back porch
	1280,	//line width
	792,	//frame height
	5120,	//minimum bytes per line based on 128 byte burst size
	0		//interlaced format
};

Xint32 DISPLAY_CONTROLLER_setInterlaced(Xuint32 baseaddr, Xint32 val)
{
	Xuint32 control = DISPLAY_CONTROLLER_mReadReg(baseaddr, DISPLAY_CONTROLLER_CONTROL_OFFSET);
	control &= ~DISPLAY_CONTROLLER_INTERLACED_MASK;
	control |= (0x1 & val) << 29;
	DISPLAY_CONTROLLER_mWriteReg(baseaddr, DISPLAY_CONTROLLER_CONTROL_OFFSET, control);
	
	return 0;
}

Xint32 DISPLAY_CONTROLLER_setResolution(Xuint32 baseaddr, Xint32 res)
{
	DISPLAY_CONTROLLER_mWriteReg(baseaddr, DISPLAY_CONTROLLER_HSYNC_OFFSET,
		ResolutionBuffer[res][0]<<16 |
		ResolutionBuffer[res][1]<<8 |
		ResolutionBuffer[res][2]);
	DISPLAY_CONTROLLER_mWriteReg(baseaddr, DISPLAY_CONTROLLER_VSYNC_POL_OFFSET,
		ResolutionBuffer[res][3]<<31 |
		ResolutionBuffer[res][10]<<24 |
		ResolutionBuffer[res][4]<<16 |
		ResolutionBuffer[res][5]<<8 |
		ResolutionBuffer[res][6]);
	DISPLAY_CONTROLLER_mWriteReg(baseaddr, DISPLAY_CONTROLLER_FRAME_SIZE_OFFSET,
		ResolutionBuffer[res][7]<<16 |
		ResolutionBuffer[res][8]);
	
	if (ResolutionBuffer[res][10])
		DISPLAY_CONTROLLER_setInterlaced(baseaddr, 1);
	else
		DISPLAY_CONTROLLER_setInterlaced(baseaddr, 0);
		
	return 0;
}

Xint32 DISPLAY_CONTROLLER_setBufferAddresses(Xuint32 baseaddr, Xint32 res, Xuint32 frameBaseAddr)
{
	Xuint32 frameOffset = ResolutionBuffer[res][8] * ResolutionBuffer[res][9];
	Xuint32 frame1Addr = frameBaseAddr + frameOffset;
	Xuint32 frame2Addr = frame1Addr + frameOffset;
	
	//xil_printf("Display Controller Frame Buffer 0 base address 0x%08x\r\n", frameBaseAddr);
	//xil_printf("Display Controller Frame Buffer 1 base address 0x%08x\r\n", frame1Addr);
	//xil_printf("Display Controller Frame Buffer 2 base address 0x%08x\r\n", frame2Addr);
	
	DISPLAY_CONTROLLER_mWriteReg(baseaddr, DISPLAY_CONTROLLER_FRAME0_BASEADDR_OFFSET, frameBaseAddr);
	DISPLAY_CONTROLLER_mWriteReg(baseaddr, DISPLAY_CONTROLLER_FRAME_ADDR_OFFSET, frameOffset);
	DISPLAY_CONTROLLER_mWriteReg(baseaddr, DISPLAY_CONTROLLER_STRIDE_OFFSET, ResolutionBuffer[res][9]);
	
	return 0;
}

Xint32 DISPLAY_CONTROLLER_setReset(Xuint32 baseaddr, Xint32 reset)
{
	Xuint32 control = DISPLAY_CONTROLLER_mReadReg(baseaddr, DISPLAY_CONTROLLER_CONTROL_OFFSET);
	control &= ~DISPLAY_CONTROLLER_RESET_MASK;
	control |= (0x1 & reset) << 31;
	DISPLAY_CONTROLLER_mWriteReg(baseaddr, DISPLAY_CONTROLLER_CONTROL_OFFSET, control);
	
	return 0;
}

Xint32 DISPLAY_CONTROLLER_setTest(Xuint32 baseaddr, Xint32 test)
{
	Xuint32 control = DISPLAY_CONTROLLER_mReadReg(baseaddr, DISPLAY_CONTROLLER_CONTROL_OFFSET);
	control &= ~DISPLAY_CONTROLLER_TEST_MASK;
	control |= (0x1 & test) << 30;
	DISPLAY_CONTROLLER_mWriteReg(baseaddr, DISPLAY_CONTROLLER_CONTROL_OFFSET, control);
	
	return 0;
}

Xint32 DISPLAY_CONTROLLER_configAll(Xuint32 baseaddr, Xint32 res, Xuint32 frameBaseAddr)
{
	DISPLAY_CONTROLLER_setResolution(baseaddr, res);
	DISPLAY_CONTROLLER_setBufferAddresses(baseaddr, res, frameBaseAddr);
	
	return 0;
}
