// DISCLAIMER OF LIABILITY
// 
// This text/file contains proprietary, confidential
// information of Xilinx, Inc., is distributed under license
// from Xilinx, Inc., and may be used, copied and/or
// disclosed only pursuant to the terms of a valid license
// agreement with Xilinx, Inc. Xilinx hereby grants you a 
// license to use this text/file solely for design, simulation, 
// implementation and creation of design files limited 
// to Xilinx devices or technologies. Use with non-Xilinx 
// devices or technologies is expressly prohibited and 
// immediately terminates your license unless covered by
// a separate agreement.
//
// Xilinx is providing this design, code, or information 
// "as-is" solely for use in developing programs and 
// solutions for Xilinx devices, with no obligation on the 
// part of Xilinx to provide support. By providing this design, 
// code, or information as one possible implementation of 
// this feature, application or standard, Xilinx is making no 
// representation that this implementation is free from any 
// claims of infringement. You are responsible for 
// obtaining any rights you may require for your implementation. 
// Xilinx expressly disclaims any warranty whatsoever with 
// respect to the adequacy of the implementation, including 
// but not limited to any warranties or representations that this
// implementation is free from claims of infringement, implied 
// warranties of merchantability or fitness for a particular 
// purpose.
//
// Xilinx products are not intended for use in life support
// appliances, devices, or systems. Use in such applications is
// expressly prohibited.
//
// Any modifications that are made to the Source Code are 
// done at the users sole risk and will be unsupported.
//
//
// Copyright (c) 2007, 2008 Xilinx, Inc. All rights reserved.
//
// This copyright and support notice must be retained as part 
// of this text at all times. 
//
//******************************************************************************/
// Filename            : vsk_dvi_out_edid.c
// $Revision:: 2237   $: Revision of last commit
// $Date:: 2008-05-09#$: Date of last commit
// Description         : VSK DVI output EDID driver file
//******************************************************************************/

#include "vsk_dvi_out_edid.h"


/******************************************************************************
* This function reads the EDID structures from the dvi_out port
*  and displays the results to the hyperterminal.
*
* @param    None.
*
* @return   None.
*
* @note     None.
*
******************************************************************************/
void vsk_dvi_out_edid_read()
{
  Xint32 i;
  Xint32 temp;
  Xuint8 num_bytes;
  Xuint8 xdata;
  Xuint8 mux_data;
  Xint32 dvi_out_pres;

  xil_printf("--DVI OUT EDID--");
    
  // See if DVI_OUT is present before trying to read it.
  dvi_out_pres = vsk_dvi_out_display_present(0);
  
  if (dvi_out_pres == 0)
  {
    print("\r\n\r\n DVI Out Display is not present.\r\n");
    print(" Please verify that the Display is properly connected.\r\n\r\n");
  }
  else
  {
    // Setup IIC_BASE_MUX for DVI OUT EDID access
    mux_data = 0x07;
    num_bytes = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_BASE_MUX_ADDR, 
                              mux_data, &mux_data, 1);
    if (num_bytes < 1) 
    { 
      print("\r\nError writing to iic_base_mux when trying to read device<iic_edid_dvi_in>\r\n");
    }
    else
    {
      for (i=0;i<256; i++)// EDID Structure
      {
        if ((i%16)==0) { xil_printf("\n\r"); }
        num_bytes = vsk_iic_read(VIDEO_OUT_IIC_BASE_ADDRESS, 
                                 IIC_EDID_DVI_OUT_ADDR, i, &xdata, 1);
        temp = (int)xdata;
        if (num_bytes == 1) { xil_printf(" %2x",temp); }
        else 
        { 
          print("\n\r-- There was an Error Reading the EDID Prom of the DVI Output.\n\r");
          print("-- Make sure that the DVI Display is properly installed.\n\r");
          break;
        }
      }
    }
    // Close the Base IIC Mux
    mux_data = 0x00;
    num_bytes = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_BASE_MUX_ADDR, 
                              mux_data, &mux_data, 1);
    if (num_bytes < 1) 
    { 
      print("\r\nError writing to iic_base_mux when trying to read device<iic_edid_dvi_in>\r\n");
    }    
  }
  xil_printf("\n\r\n\r");        
        
}
