// DISCLAIMER OF LIABILITY
// 
// This text/file contains proprietary, confidential
// information of Xilinx, Inc., is distributed under license
// from Xilinx, Inc., and may be used, copied and/or
// disclosed only pursuant to the terms of a valid license
// agreement with Xilinx, Inc. Xilinx hereby grants you a 
// license to use this text/file solely for design, simulation, 
// implementation and creation of design files limited 
// to Xilinx devices or technologies. Use with non-Xilinx 
// devices or technologies is expressly prohibited and 
// immediately terminates your license unless covered by
// a separate agreement.
//
// Xilinx is providing this design, code, or information 
// "as-is" solely for use in developing programs and 
// solutions for Xilinx devices, with no obligation on the 
// part of Xilinx to provide support. By providing this design, 
// code, or information as one possible implementation of 
// this feature, application or standard, Xilinx is making no 
// representation that this implementation is free from any 
// claims of infringement. You are responsible for 
// obtaining any rights you may require for your implementation. 
// Xilinx expressly disclaims any warranty whatsoever with 
// respect to the adequacy of the implementation, including 
// but not limited to any warranties or representations that this
// implementation is free from claims of infringement, implied 
// warranties of merchantability or fitness for a particular 
// purpose.
//
// Xilinx products are not intended for use in life support
// appliances, devices, or systems. Use in such applications is
// expressly prohibited.
//
// Any modifications that are made to the Source Code are 
// done at the users sole risk and will be unsupported.
//
//
// Copyright (c) 2007, 2008 Xilinx, Inc. All rights reserved.
//
// This copyright and support notice must be retained as part 
// of this text at all times. 
//
//******************************************************************************/
 // Filename            : vsk_util.c
// $Revision:: 3508   $: Revision of last commit
// $Date:: 2008-09-04#$: Date of last commit
// Description         : VSK Utility functions file
//******************************************************************************/

#include "vsk_util.h"


/***********************************************************************
* Wait the specified number of microseconds
*
* @param    None.
*
* @return   None. 
*
* @note     Calibrated at 100MHz CPU clock using scope, 
*            should work for any CPU speed
*
***********************************************************************/
void usec_wait(Xint32 delay) {
	Xint32 val = delay * (Xint32)( (XPAR_MICROBLAZE_CORE_CLOCK_FREQ_HZ * 17) / 100000000 );
	while(val--) 
		asm("nop");
}

/***********************************************************************
* This function puts the dvi output chrontel chip into reset
*
* @param    None.
*
* @return   None. 
*
* @note     None.
*
***********************************************************************/
void set_chrontel_reset_n()
{
  Xuint32 value;

  value = XIo_In32(VIDEO_OUT_IIC_BASE_ADDRESS + 0x124);
  value = value & 0xFFFFFFFE; // Force bit 0 to 0
  XIo_Out32((VIDEO_OUT_IIC_BASE_ADDRESS + 0x124),value);
}

/***********************************************************************
* This function releases the dvi output chrontel chip from reset
*
* @param    None.
*
* @return   None. 
*
* @note     None.
*
***********************************************************************/
void release_chrontel_reset_n()
{
  Xuint32 value;

  value = XIo_In32(VIDEO_OUT_IIC_BASE_ADDRESS + 0x124);
  value = value | 0x00000001; // Force bit 0 to 1
  XIo_Out32((VIDEO_OUT_IIC_BASE_ADDRESS + 0x124),value);
}

/***********************************************************************
* This function puts the DCM_0 PCORE into reset
*
* @param    None.
*
* @return   None. 
*
* @note     None.
*
***********************************************************************/
void set_dcm_0_reset()
{
  Xuint32 value;

//  value = XIo_In32(FMC_CARD_IIC_BASE_ADDRESS + 0x124);
//  value = value | 0x00000001; // Force bit 0 to 1
//  XIo_Out32((FMC_CARD_IIC_BASE_ADDRESS + 0x124),value);
}

/***********************************************************************
* This function releases the DCM_0 PCORE from reset
*
* @param    None.
*
* @return   None. 
*
* @note     None.
*
***********************************************************************/
void release_dcm_0_reset()
{
  Xuint32 value;

//  value = XIo_In32(FMC_CARD_IIC_BASE_ADDRESS + 0x124);
//  value = value & 0xFFFFFFFE; // Force bit 0 to 0
//  XIo_Out32((FMC_CARD_IIC_BASE_ADDRESS + 0x124),value);
}

/***********************************************************************
* This function puts the dvi_in PCORE into Digital Mode
*
* @param    None.
*
* @return   None. 
*
* @note     None.
*
***********************************************************************/
void set_dvi_in_mode_digital()
{
  Xuint32 value;

//  value = XIo_In32(FMC_CARD_IIC_BASE_ADDRESS + 0x124);
//  value = value & 0xFFFFFFFD; // Force bit 1 to 0
//  XIo_Out32((FMC_CARD_IIC_BASE_ADDRESS + 0x124),value);
}

/***********************************************************************
* This function puts the dvi_in PCORE into Analog Mode
*
* @param    None.
*
* @return   None. 
*
* @note     None.
*
***********************************************************************/
void set_dvi_in_mode_analog()
{
  Xuint32 value;

//  value = XIo_In32(FMC_CARD_IIC_BASE_ADDRESS + 0x124);
//  value = value | 0x00000002; // Force bit 1 to 1
//  XIo_Out32((FMC_CARD_IIC_BASE_ADDRESS + 0x124),value);
}
