//////////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 2014 UTIA AV CR v.v.i. All rights reserved.
//
// UTIA IS PROVIDING THIS CODE, "AS IS" AS A COURTESY TO YOU.
// BY PROVIDING THIS CODE, AS ONE POSSIBLE IMPLEMENTATION,
// UTIA IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
// IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
// FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
// UTIA EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
// THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
// ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
// FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.
//
//////////////////////////////////////////////////////////////////////////


#include "xparameters.h"
#include "stdio.h"
#include "string.h"
#include "platform.h"

#include "xilmfs.h"
#include "mfs_config.h"

#include "wal.h"
#include "wal_bce_jk.h"
#include "bce_fp11_1x8_0_plbw.h"

WAL_REGISTER_WORKER(worker1, BCE_JK_FP32M24, BCE_FP11_1X8_0_PLBW_ConfigTable, 0,
		8, 0);

#include "fill_FA1101P0_program_store.h"
#include "fill_FA1101P1_program_store.h"

#include "fill_FP1101P0_program_store.h"
#include "fill_FP1101P1_program_store.h"

#define NN  (10)
#define ERR (0.0f)

void draw_pb2mb_introduction(wal_worker_t *wrk, int fw);
void bce_fp11_1x8_VADD_introduction(wal_worker_t *wrk, int fw,
		unsigned int capab, unsigned int ah, unsigned int bh, unsigned int zh);

int main() {
	unsigned int ah, bh, zh, i;
	unsigned int capabilities1;
	unsigned int license1;

	int fw;
	int num_blocks_used;
	int num_blocks_free;

	xil_printf("\n\r\n\r");

	xil_printf("***********************************\n\r");
	xil_printf("***********************************\n\r");
	xil_printf("**   UTIA   EdkDSP evaluation    **\n\r");
	xil_printf("**   Xilinx SP605 board, PLB bus **\n\r");
	xil_printf("***********************************\n\r");
	xil_printf("***********************************\n\r\n\r");

	if (init_platform() < 0) {
		xil_printf("ERROR initializing platform.\r\n");
		return -1;
	}

	if (wal_init_worker(worker1)) {
		xil_printf("\r\n ERROR: Init wrk failed.\r\n");
		return 0;
	}

	mfs_delete_file("FP1101.TXT");

	fw = mfs_file_open("FP1101.TXT", MFS_MODE_CREATE);
	if (fw != -1) {
		xil_printf("\r\nFile FP1101.TXT created for wr");
	} else {
		xil_printf("\r\nFailed to create File FP1101.TXT open for wr.");
	}

	///////////////////////////////////////////////
	// Upgrade firmware
	for (i = 0; i < 3; i++) {
		fill_FP1101P0_program_store[i] = fill_FA1101P0_program_store[i];
		fill_FP1101P1_program_store[i] = fill_FA1101P1_program_store[i];
	}
	for (i = EDKDSP_BOOT; i < WAL_FW_FULL_SIZE; i++) {
		fill_FP1101P0_program_store[i] = fill_FA1101P0_program_store[i];
		fill_FP1101P1_program_store[i] = fill_FA1101P1_program_store[i];
	}

	xil_printf("\r\nTests of vector operations:");

	wal_set_firmware(worker1, WAL_PBID_P0, fill_FP1101P0_program_store, -1);
	wal_set_firmware(worker1, WAL_PBID_P1, fill_FP1101P1_program_store, -1);

	//Worker code related to wal_get_capabilities() and wal_get_license()
	//08		op = mb2pb_read_data();
	//
	//10		pb2dfu_set(C_DFU_CNT, 0);
	//11		pb2dfu_set(C_DFU_OP, op);
	//12		pb2dfu_wait4hw();
	//			...
	//100		pb2mb_eoc('.');
	//101		pb2mb_req_reset('.');
	//102		pb2mb_reset();

	wal_get_capabilities(worker1, WAL_PBID_P0, &capabilities1);
	xil_printf("\r\nPB:HW Operations: Code = %x", capabilities1);
	wal_get_license(worker1, WAL_PBID_P0, &license1);

	if (license1) {
		for (ah = 0; ah < 4; ah++) {
			for (bh = 0; bh < 4; bh++) {
				for (zh = 0; zh < 4; zh++) {
					bce_fp11_1x8_VADD_introduction(worker1, fw, capabilities1, ah, bh, zh);
				}
			}
		}
	} else {
		xil_printf("\r\n");
		xil_printf("\r\n Evaluation license will expire soon. Please contact:");
		xil_printf("\r\n UTIA AV CR, v.v.i.; Department of Signal Processing");
		xil_printf("\r\n Pod vodarenskou vezi 4");
		xil_printf("\r\n 18208, Prague 8, Czech republic");
		xil_printf("\r\n http://zs.utia.cas.cz \r\n");
	}

	mfs_file_close(fw);
	mfs_get_usage(&num_blocks_used, &num_blocks_free);
	xil_printf("\r\n\r\nBlocks_used %d", num_blocks_used);
	xil_printf("\r\nBlocks_free %d", num_blocks_free);
	xil_printf("\r\n\r\n");
	mfs_ls();
	return 0;
}

void bce_fp11_1x8_VADD_introduction(wal_worker_t *wrk, int fw,
		unsigned int capab, unsigned int ah, unsigned int bh, unsigned int zh) {
	float af[8][NN], bf[8][NN], zfr[8][NN];
	float r, err;
	unsigned int i, j, k;
	unsigned int a = 0, as = 0, b = 0, bs = 0, z = 0, zs = 0;
	unsigned int inca = 1, incb = 1, incz = 1;
	unsigned int nn, op;

	if (capab & WAL_BCE_JK_CAP_VADD) {
		nn = NN;
		xil_printf("\r\n");
		xil_printf("\r\nTest VADD '%s' ...............", wrk->name);
		op = WAL_BCE_JK_VADD;

		for (i = 0; i < nn; i++) {
			for (k = 0; k < 8; k++) {
				af[k][i] = (float) i + (float) k + (float) 0.123456;
				bf[k][i] = (float) i + (float) k + (float) 0.789;
			}
		}

		for (k = 0; k < 8; k++) {
			wal_mb2dmem(wrk, k, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[k][0], nn);
			wal_mb2dmem(wrk, k, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[k][0], nn);
		}

		//Worker code related to wal_bce_jk_create_operation()
		//08			op = mb2pb_read_data();
		//
		//14			pb2dfu_set(C_DFU_A_MEM_ADDR, mb2pb_read_data());
		//15			pb2dfu_set(C_DFU_A_MEM_SADDR, mb2pb_read_data());
		//16			pb2dfu_set(C_DFU_B_MEM_ADDR, mb2pb_read_data());
		//17			pb2dfu_set(C_DFU_B_MEM_SADDR, mb2pb_read_data());
		//18			pb2dfu_set(C_DFU_Z_MEM_ADDR, mb2pb_read_data());
		//19			pb2dfu_set(C_DFU_Z_MEM_SADDR, mb2pb_read_data());
		//20			ah = mb2pb_read_data();
		//21			pb2dfu_set(C_DFU_A_MEM_BANK, ah);
		//22			bh = mb2pb_read_data();
		//23			pb2dfu_set(C_DFU_B_MEM_BANK, bh);
		//24			zh = mb2pb_read_data();
		//25			pb2dfu_set(C_DFU_Z_MEM_BANK, zh);
		//26			pb2dfu_set(C_DFU_A_INC, mb2pb_read_data());
		//27			pb2dfu_set(C_DFU_B_INC, mb2pb_read_data());
		//28			pb2dfu_set(C_DFU_Z_INC, mb2pb_read_data());
		//29			n = mb2pb_read_data() - 1;
		//30			pb2dfu_set(C_DFU_CNT, n);
		//31			...
		//32			pb2dfu_set(C_DFU_OP, op);
		//33			{
		//34			...
		//35			}
		//36			pb2dfu_wait4hw();

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);

		xil_printf("\r\nWR to FP1101.TXT from '%s' ...", wrk->name);
		draw_pb2mb_introduction(wrk, fw);
		xil_printf("\r\nWR to FP1101.TXT from '%s' ... OK", wrk->name);

		//Worker code related to wal_bce_jk_sync_operation()
		//100		pb2mb_eoc('.');
		//101		pb2mb_req_reset('.');
		//102		pb2mb_reset();

		wal_bce_jk_sync_operation(wrk);

		for (k = 0; k < 8; k++) {
			wal_dmem2mb(wrk, k, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zfr[k][0],
					nn);
		}

		j = 0;

		for (k = 0; k < 8; k++) {
			for (i = 0; i < nn; i++) {
				r = af[k][i] + bf[k][i];
				err = zfr[k][i] - r;
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VADD Error: zfr[%d][%d] = %d   r = %d", k, i,
						(unsigned int) zfr[k][i], (unsigned int) r);
				j++;
			}
		}
	}
		if (j == 0) {
			xil_printf("\r\nTest VADD '%s' ............... OK", wrk->name);
		}
	} else {
		xil_printf("\r\n VADD is not supported.");
	}
}


void draw_pb2mb_introduction(wal_worker_t *wrk, int fw) {
	char buf[32];
	uint32_t status_led, status_screen;

	wal_pb2mb(wrk, &status_screen);
	while ((status_screen & 0x7) != 0) {

		strcpy(buf, "\r\n Screen0 ");
		xil_printf("\r\n Screen");
		buf[9] = (char) ((status_screen & 0xFF) + 0x30);
		xil_printf("%c\r\n", (char) buf[9]);
		mfs_file_write(fw, buf, 10);

		wal_pb2mb(wrk, &status_led);
		while ((status_led & WAL_STAT_EXIT_BIT)== 0){
			buf[0] = (char) (status_led & 0xFF);
			xil_printf("%c", (char) buf[0]);
			mfs_file_write(fw, buf, 1);

			wal_pb2mb(wrk, &status_led);
		}
		wal_pb2mb(wrk, &status_screen);
	}
	strcpy(buf, "\r\n Screen0 ");
	xil_printf("\r\n Screen");
	buf[9] = (char) ((status_screen & 0xFF) + 0x30);
	xil_printf("%c\r\n", (char) buf[9]);
	mfs_file_write(fw, buf, 10);
	wal_pb2mb(wrk, &status_led);
	while ((status_led & WAL_STAT_EXIT_BIT)== 0){
		buf[0] = (char) (status_led & 0xFF);
		xil_printf("%c", (char) buf[0]);
		mfs_file_write(fw, buf, 1);
		wal_pb2mb(wrk, &status_led);
	}
}
