// DISCLAIMER OF LIABILITY
// 
// This text/file contains proprietary, confidential
// information of Xilinx, Inc., is distributed under license
// from Xilinx, Inc., and may be used, copied and/or
// disclosed only pursuant to the terms of a valid license
// agreement with Xilinx, Inc. Xilinx hereby grants you a 
// license to use this text/file solely for design, simulation, 
// implementation and creation of design files limited 
// to Xilinx devices or technologies. Use with non-Xilinx 
// devices or technologies is expressly prohibited and 
// immediately terminates your license unless covered by
// a separate agreement.
//
// Xilinx is providing this design, code, or information 
// "as-is" solely for use in developing programs and 
// solutions for Xilinx devices, with no obligation on the 
// part of Xilinx to provide support. By providing this design, 
// code, or information as one possible implementation of 
// this feature, application or standard, Xilinx is making no 
// representation that this implementation is free from any 
// claims of infringement. You are responsible for 
// obtaining any rights you may require for your implementation. 
// Xilinx expressly disclaims any warranty whatsoever with 
// respect to the adequacy of the implementation, including 
// but not limited to any warranties or representations that this
// implementation is free from claims of infringement, implied 
// warranties of merchantability or fitness for a particular 
// purpose.
//
// Xilinx products are not intended for use in life support
// appliances, devices, or systems. Use in such applications is
// expressly prohibited.
//
// Any modifications that are made to the Source Code are 
// done at the user?s sole risk and will be unsupported.
//
//
// Copyright (c) 2007 - 2009 Xilinx, Inc. All rights reserved.
//
// This copyright and support notice must be retained as part 
// of this text at all times. 
//
#include <stdio.h>

// Located in: microblaze_0/include/
#include "xparameters.h"
#include "vsk_iic.h"
#include "vsk_iic_diag.h"
#include "vsk_dvi_out.h"
#include "vsk_iic_clock.h"
#include "vsk_util.h"
#include "display_controller.h"
#include "xstatus.h"
#include "xuartlite.h"

#include "platform_config.h"

#ifdef NOTNOW
#define DISPLAYCON_ADDR			XPAR_DISPLAY_CONTROLLER_0_BASEADDR
#else
#define DISPLAYCON_ADDR			XPAR_DISPLAY_CNTLR_BASEADDR
#endif

Xuint32 frameBufferBaseAddr;

static void
output_res_1024x768 (Xuint32 frameBufferBaseAddr)
{
    DISPLAY_CONTROLLER_setReset(DISPLAYCON_ADDR, 1);
    DISPLAY_CONTROLLER_configAll(DISPLAYCON_ADDR,
                                 DISPLAY_RESOLUTION_XGA, frameBufferBaseAddr);
    DISPLAY_CONTROLLER_setReset(DISPLAYCON_ADDR, 0);
    vsk_dvi_out_config(IIC_DVI_OUT_1024x768P70);
    print("Output resolution set to 1024x768p70\r\n");
}

static void
output_res_800x600 (Xuint32 frameBufferBaseAddr)
{
    DISPLAY_CONTROLLER_setReset(DISPLAYCON_ADDR, 1);
    DISPLAY_CONTROLLER_configAll(DISPLAYCON_ADDR,
                                 DISPLAY_RESOLUTION_SVGA, frameBufferBaseAddr);
    DISPLAY_CONTROLLER_setReset(DISPLAYCON_ADDR, 0);
    vsk_dvi_out_config(IIC_DVI_OUT_800x600P60);
    print("Output resolution set to 800x600p60\r\n");
}

static void
output_res_1280x720 (Xuint32 frameBufferBaseAddr)
{
    DISPLAY_CONTROLLER_setReset(DISPLAYCON_ADDR, 1);
    DISPLAY_CONTROLLER_configAll(DISPLAYCON_ADDR,
                                 DISPLAY_RESOLUTION_720P60,
                                 frameBufferBaseAddr);
    DISPLAY_CONTROLLER_setReset(DISPLAYCON_ADDR, 0);
    vsk_dvi_out_config(IIC_DVI_OUT_720P60);
    print("Output resolution set to 1280x720p\r\n");
}

static void
output_res_1280x792 (Xuint32 frameBufferBaseAddr)
{
    DISPLAY_CONTROLLER_setReset(DISPLAYCON_ADDR, 1);
    DISPLAY_CONTROLLER_configAll(DISPLAYCON_ADDR,
    							 DISPLAY_RESOLUTION_UTIA_ANA,
                                 frameBufferBaseAddr);
    DISPLAY_CONTROLLER_setReset(DISPLAYCON_ADDR, 0);
    vsk_dvi_out_config(IIC_DVI_OUT_UTIA_ANA);
    print("Output resolution set to 1280x792p\r\n");
}

static void
output_res_1920x1080 (Xuint32 frameBufferBaseAddr)
{
    DISPLAY_CONTROLLER_setReset(DISPLAYCON_ADDR, 1);
    DISPLAY_CONTROLLER_configAll(DISPLAYCON_ADDR,
    							 DISPLAY_RESOLUTION_1080I60,
                                 frameBufferBaseAddr);
    DISPLAY_CONTROLLER_setReset(DISPLAYCON_ADDR, 0);
    vsk_dvi_out_config(IIC_DVI_OUT_1080I60);
    print("Output resolution set to 1920x1080i60\r\n");
}

XStatus
vsk_setup (void)
{
  // Initialize the IIC cores
  vsk_iic_init();

  // Put DVI_OUT into reset
  set_chrontel_reset_n();

  // Initialize the DVI_OUT
  release_chrontel_reset_n();
  vsk_dvi_out_display_present(1);

   frameBufferBaseAddr = UTIASPEDK_FRAME_BUFFER_BASE_ADDR;
  // Set output resolution 
  if (VIDEO_X_RES == 1920) {
      output_res_1920x1080(frameBufferBaseAddr);
  }
  if (VIDEO_X_RES == 1280) {
        output_res_1280x792(frameBufferBaseAddr);
  }
  if (VIDEO_X_RES == 1024){
      output_res_1024x768(frameBufferBaseAddr);
  }

  // Set the frame buffer addresss 
  DISPLAY_CONTROLLER_mWriteReg(DISPLAYCON_ADDR,
                               DISPLAY_CONTROLLER_FRAME0_BASEADDR_OFFSET,
                               frameBufferBaseAddr);
  DISPLAY_CONTROLLER_mWriteReg(DISPLAYCON_ADDR,
                               DISPLAY_CONTROLLER_FRAME_ADDR_OFFSET,
                               0);

  // Start out in pass-through mode
  //vsk_disable_fir_2d_coef_update();
  //vsk_load_fir_identity();
  //vsk_enable_fir_2d_coef_update();

  return 0;
} 

