// DISCLAIMER OF LIABILITY
// 
// This text/file contains proprietary, confidential
// information of Xilinx, Inc., is distributed under license
// from Xilinx, Inc., and may be used, copied and/or
// disclosed only pursuant to the terms of a valid license
// agreement with Xilinx, Inc. Xilinx hereby grants you a 
// license to use this text/file solely for design, simulation, 
// implementation and creation of design files limited 
// to Xilinx devices or technologies. Use with non-Xilinx 
// devices or technologies is expressly prohibited and 
// immediately terminates your license unless covered by
// a separate agreement.
//
// Xilinx is providing this design, code, or information 
// "as-is" solely for use in developing programs and 
// solutions for Xilinx devices, with no obligation on the 
// part of Xilinx to provide support. By providing this design, 
// code, or information as one possible implementation of 
// this feature, application or standard, Xilinx is making no 
// representation that this implementation is free from any 
// claims of infringement. You are responsible for 
// obtaining any rights you may require for your implementation. 
// Xilinx expressly disclaims any warranty whatsoever with 
// respect to the adequacy of the implementation, including 
// but not limited to any warranties or representations that this
// implementation is free from claims of infringement, implied 
// warranties of merchantability or fitness for a particular 
// purpose.
//
// Xilinx products are not intended for use in life support
// appliances, devices, or systems. Use in such applications is
// expressly prohibited.
//
// Any modifications that are made to the Source Code are 
// done at the users sole risk and will be unsupported.
//
//
// Copyright (c) 2007, 2008 Xilinx, Inc. All rights reserved.
//
// This copyright and support notice must be retained as part 
// of this text at all times. 
//
//******************************************************************************/
// Filename            : vsk_iic_clock.c
// $Revision:: 2183   $: Revision of last commit
// $Date:: 2008-05-05#$: Date of last commit
// Description         : Implementation of IIC programmable clock driver.
//******************************************************************************/

/*****************************************************************************/
/*
* @file vsk_iic_clock.c
*
* This file consists of a polled mode design example which uses the Xilinx
* IIC peripheral in dynamic mode and its low-level driver to access the
* IIC interface of the clock generator chip on the Spartan-3A DSP 3400A board.
*
* The clock generator is at address 0x6A on the IIC_SCL_MAIN, IIC_SDA_MAIN
* IIC chain.  Jumper JP6 must be removed for IIC access to the clock generator.
*
* @note
*
* This file was derived from iic_clock.c which was developed to support ML50X.
*
* <pre>
* </pre>
*
******************************************************************************/

#include "vsk_iic_clock_l.h"

/************************** Variable Definitions *****************************/

// The data in the follwing array are only used if NEEDS_INIT is enabled.
static Xuint8 SetupBuffer[NUM_SETUP_ADDR][2] = {
// Addr, Data
	0x34, 0x46,
	0x35, 0xd9,
	0x4d, 0xb3,
	0x4e, 0x0a,
	0x5d, 0xbb,
	0x5e, 0x04
};

static Xuint8 ConfigBuffer[NUM_CLOCK_CONFIG][NUM_CONFIG_ADDR][2] = {
// 27MHz config
// Addr, Data
	0x38, 0x56,
	0x39, 0x56, // copy of above, required? 
	0x3C, 0xA5,
	0x3D, 0xA5, // copy of above, required? 
	0x40, 0x05,
	0x41, 0x05, // copy of above, required? 
	0x44, 0xd8,
	0x45, 0xd8, // copy of above, required?
	0x48, 0x00,
	0x49, 0x00,
	0x5D, 0xBB, 
	0x5E, 0x04, 
// 25.175MHz config
// Addr, Data
	0x38, 0x1f,
	0x39, 0x1f, // copy of above, required?
	0x3C, 0xff,
	0x3D, 0xff, // copy of above, required? 
	0x40, 0x19,
	0x41, 0x19, // copy of above, required?
	0x44, 0xef,
	0x45, 0xef, // copy of above, required?
	0x48, 0x03,
	0x49, 0x03,
	0x5D, 0xBB, 
	0x5E, 0x04, 
// 74.25MHz config
// Addr, Data
	0x38, 0x3e,
	0x39, 0x3e,
	0x3C, 0x74,
	0x3D, 0x74,
	0x40, 0x0a,
	0x41, 0x0a,
	0x44, 0x29,
	0x45, 0x29,
	0x48, 0x01,
	0x49, 0x01,
	0x5D, 0xBF,
	0x5E, 0x00,
// 40.00MHz config
// Addr, Data
	0x38, 0x63,
	0x39, 0x63,
	0x3C, 0x22,
	0x3D, 0x22,
	0x40, 0x01,
	0x41, 0x01,
	0x44, 0x30,
	0x45, 0x30,
	0x48, 0x00,
	0x49, 0x00,
	0x5D, 0xB7,
	0x5E, 0x03,
// 65.00MHz config
// Addr, Data
	0x38, 0x62,
	0x39, 0x62,
	0x3C, 0x63,
	0x3D, 0x63,
	0x40, 0x01,
	0x41, 0x01,
	0x44, 0x1A,
	0x45, 0x1A,
	0x48, 0x00,
	0x49, 0x00,
	0x5D, 0xBF,
	0x5E, 0x00
};

/************************** Function Definitions *****************************/

/*****************************************************************************/
/**
* Main function to call the low level Clock example.
*
* @param    None.
*
* @return   XST_SUCCESS if successful else XST_FAILURE.
*
* @note     None.
*
******************************************************************************/
XStatus vsk_iic_clock_test (void)
{
   Xuint8 mux_data;
   XStatus TestStatus=XST_FAILURE;
   Xuint8 byte_count;
	
	mux_data = 0x05; // select mux channel 1
   byte_count = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_BASE_MUX_ADDR, 
                            mux_data, &mux_data, 1);
	
#ifdef DEBUG
	if(byte_count < 1) {
		print("Could not write to carrier board I2C mux.\r\n");
	}

    print("IIC Dynamic mode CLOCK GENERATOR read back -\r\n");
#endif
    /*
     * Initialize the IIC Core.
     */
    TestStatus = XIicLowLevelClockInit();
    if (TestStatus != XST_SUCCESS)
    {
#ifdef DEBUG
      print("Test failed (during initialization)\r\n");
#endif
	return XST_FAILURE;
    }

    /*
     * Perform the Read operation.
     */
    TestStatus = XIicLowLevelClockDump();
    if (TestStatus != XST_SUCCESS)
    {
#ifdef DEBUG
      print("Test failed\r\n");
#endif
        return XST_FAILURE;
    }

#ifdef DEBUG
    print ("Test passed\r\n");
#endif
	return XST_SUCCESS;
}

/******************************************************************************
* The function uses the low level driver of IIC to read from the IIC device 
*
* @param    None.
*
* @return   XST_SUCCESS if successful else XST_FAILURE.
*
* @note     None.
*
******************************************************************************/
XStatus XIicLowLevelClockDump()
{
    Xuint8 BytesRead;
    Xuint8 BytesWritten;
    Xuint8 StatusReg;
    Xuint8 Index;
	 Xuint8 ReadVal;

    /*
     * Make sure all the Fifo's are cleared and Bus is Not busy.
     */
    do
    {
         StatusReg = XIo_In8(IIC_BASE_ADDRESS + XIIC_SR_REG_OFFSET);
         StatusReg = StatusReg & (XIIC_SR_RX_FIFO_EMPTY_MASK |
			          XIIC_SR_TX_FIFO_EMPTY_MASK |
                                  XIIC_SR_BUS_BUSY_MASK);
    }while (StatusReg != (XIIC_SR_RX_FIFO_EMPTY_MASK |
			  XIIC_SR_TX_FIFO_EMPTY_MASK));

    /*
     * Display read values. 
     */
    for (Index = 0; Index < PAGE_SIZE; Index++)
    {
		/*
		 * Read from the Clock Generator IIC interface.
       */
		BytesRead = ClockReadOneByte(IIC_CLOCK_TEST_START_ADDRESS + Index, &ReadVal);
		if(BytesRead != 1)
		{
			return XST_FAILURE;
		}
#ifdef DEBUG
		xil_printf("iic_clock[%d] = %02X\r\n", Index, ReadVal);
#endif
    }
    return XST_SUCCESS;
}

XStatus vsk_iic_clock_config (Xuint32 curConfig)
{
	Xuint8 Index;
	Xuint8 BytesWritten;
	Xuint8 TotalBytesWritten = 0;
	Xuint8 ExpectedBytesWritten = 0;
	static Xuint8 inited = 0;
   	Xuint8 mux_data;
   	Xuint8 byte_count;
	
	mux_data = 0x05; // select mux channel 1
   byte_count = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_BASE_MUX_ADDR, 
                            mux_data, &mux_data, 1);
#ifdef DEBUG
	if(byte_count < 1) {
		print("Could not write to carrier board I2C mux.\r\n");
	}
#endif
#ifdef NEEDS_INIT
	if (!inited) {
		for (Index = 0; Index < NUM_SETUP_ADDR; Index++)
		{
			BytesWritten = ClockWriteByte(SetupBuffer[Index][0], &SetupBuffer[Index][1], 1);
#ifdef DEBUG
			xil_printf("addr=0x%x, data=0x%x\r\n", SetupBuffer[Index][0], SetupBuffer[Index][1]);
#endif
			TotalBytesWritten += BytesWritten;
		}
		ExpectedBytesWritten += NUM_SETUP_ADDR;
		inited = 1;
	}
#endif

#ifdef DEBUG
	if(curConfig == CLOCK_CONFIG_27000MHZ) {
		print("setting up CLOCK_CONFIG_27000MHZ\r\n");
	} else
		print("setting up CLOCK_CONFIG_25175MHZ\r\n");
#endif	
	for (Index = 0; Index < NUM_CONFIG_ADDR; Index++)
	{
		BytesWritten = ClockWriteByte(ConfigBuffer[curConfig][Index][0], &ConfigBuffer[curConfig][Index][1], 1);
#ifdef DEBUG
		xil_printf("addr=0x%x, data=0x%x\r\n", ConfigBuffer[curConfig][Index][0], ConfigBuffer[curConfig][Index][1]);
#endif
		TotalBytesWritten += BytesWritten;
	}
	ExpectedBytesWritten	+= NUM_CONFIG_ADDR;
	
	if (TotalBytesWritten < ExpectedBytesWritten) {
		return XST_FAILURE;
	} else 
		return XST_SUCCESS;
}

/*****************************************************************************/
/**
* This function writes a buffer of bytes to the IIC serial Clock.
*
* @param    Address contains the start address in the Clock to write to.
* @param    BufferPtr contains the address of the data to write.
* @param    ByteCount contains the number of bytes in the buffer to be written.
*           Note that this should not exceed the page size of the Clock as
*           noted by the constant PAGE_SIZE.
*
* @return   The number of bytes written, a value less than that which was
*           specified as an input indicates an error.
*
* @note     None.
*
******************************************************************************/
Xuint8 ClockWriteByte(AddressType Address, Xuint8 *BufferPtr,
					   Xuint8 ByteCount)
{
    Xuint8 SentByteCount;
    Xuint8 WriteBuffer[sizeof(Address) + 2]; // was + PAGE_SIZE
    Xuint8 Index;

    WriteBuffer[0] = 0;
    WriteBuffer[1] = (Xuint8)(Address);
	  
    /*
     * Put the data in the write buffer following the address.
     */
    for (Index = 0; Index < ByteCount; Index++)
    {
        WriteBuffer[sizeof(Address) + Index + 1] = BufferPtr[Index];
    }

    /*
     * Write a page of data at the specified address to the Clock.
     */
    SentByteCount = XIic_DynSend(IIC_BASE_ADDRESS, IIC_CLOCK_ADDR, WriteBuffer,
                                 sizeof(Address) + 2, XIIC_STOP); // was + PAGE_SIZE

    /*
     * Return the number of bytes written to the Clock.
     */
    return SentByteCount - (sizeof(Address) + 1);
}

/******************************************************************************
* This function reads one byte from the IIC serial Clock into a
* specified buffer.
*
* @param    Address contains the start address in the Clock to read from.
* @param    BufferPtr contains the address of the data buffer to be filled.
*
* @return   The number of bytes read. Zero indicates an error.
*
* @note     None.
*
******************************************************************************/
Xuint8 ClockReadOneByte(AddressType Address, Xuint8 *BufferPtr)
{
    Xuint8 ReceivedByteCount;
    Xuint8 RecvBuf[2];

	 ReceivedByteCount = ClockReadByte(Address, RecvBuf, 2);
	 
	 if(ReceivedByteCount == 2) {
		BufferPtr[0] = RecvBuf[1];
		return 1;
	 } 
	 else
		return 0;
}

/******************************************************************************
* This function reads a number of bytes from the IIC serial Clock into a
* specified buffer.
*
* @param    Address contains the start address in the Clock to read from.
* @param    BufferPtr contains the address of the data buffer to be filled.
* @param    ByteCount contains the number of bytes in the buffer to be read.
*           This value is constrained by the page size of the device such
*           that up to 64K may be read in one call.
*
* @return   The number of bytes read. A value less than the specified input
*           value indicates an error.
*
* @note     None.
*
******************************************************************************/
Xuint8 ClockReadByte(AddressType Address, Xuint8 *BufferPtr, Xuint8 ByteCount)
{
    Xuint8 ReceivedByteCount;
    Xuint8 SentByteCount;
    Xuint16 StatusReg;

    Xuint8 CommandBuf[2];

    CommandBuf[0] = 0;
    CommandBuf[1] = Address;

    /*
     * Position the Read pointer to specific location in the Clock.
     */
    do
    {

         StatusReg = XIo_In8(IIC_BASE_ADDRESS + XIIC_SR_REG_OFFSET);
         if(!(StatusReg & XIIC_SR_BUS_BUSY_MASK))
         {
              SentByteCount = XIic_DynSend(IIC_BASE_ADDRESS, IIC_CLOCK_ADDR,
                                   (Xuint8 *)CommandBuf, sizeof(CommandBuf),
                                   XIIC_REPEATED_START);
       	 }
    }while(SentByteCount != sizeof(CommandBuf));

    /*
     * Receive the data.
     */
    ReceivedByteCount = XIic_DynRecv(IIC_BASE_ADDRESS, IIC_CLOCK_ADDR,
		                     BufferPtr, ByteCount);

    /*
     * Return the number of bytes received from the Clock.
    */
    return ReceivedByteCount;
}

/******************************************************************************
* The function uses the low level driver of IIC to initialize the core.
*
* @param    None.
*
* @return   XST_SUCCESS if successful else XST_FAILURE.
*
* @note     None.
*
******************************************************************************/
XStatus XIicLowLevelClockInit()
{
   XStatus Status;

   /*
    * Initialize the IIC Core.
    */
   Status = XIic_DynInit(IIC_BASE_ADDRESS);
   if(Status != XST_SUCCESS)
   {
       return XST_FAILURE;
   }

   return XST_SUCCESS;
}

