// DISCLAIMER OF LIABILITY
// 
// This text/file contains proprietary, confidential
// information of Xilinx, Inc., is distributed under license
// from Xilinx, Inc., and may be used, copied and/or
// disclosed only pursuant to the terms of a valid license
// agreement with Xilinx, Inc. Xilinx hereby grants you a 
// license to use this text/file solely for design, simulation, 
// implementation and creation of design files limited 
// to Xilinx devices or technologies. Use with non-Xilinx 
// devices or technologies is expressly prohibited and 
// immediately terminates your license unless covered by
// a separate agreement.
//
// Xilinx is providing this design, code, or information 
// "as-is" solely for use in developing programs and 
// solutions for Xilinx devices, with no obligation on the 
// part of Xilinx to provide support. By providing this design, 
// code, or information as one possible implementation of 
// this feature, application or standard, Xilinx is making no 
// representation that this implementation is free from any 
// claims of infringement. You are responsible for 
// obtaining any rights you may require for your implementation. 
// Xilinx expressly disclaims any warranty whatsoever with 
// respect to the adequacy of the implementation, including 
// but not limited to any warranties or representations that this
// implementation is free from claims of infringement, implied 
// warranties of merchantability or fitness for a particular 
// purpose.
//
// Xilinx products are not intended for use in life support
// appliances, devices, or systems. Use in such applications is
// expressly prohibited.
//
// Any modifications that are made to the Source Code are 
// done at the users sole risk and will be unsupported.
//
//
// Copyright (c) 2007, 2008 Xilinx, Inc. All rights reserved.
//
// This copyright and support notice must be retained as part 
// of this text at all times. 
//
//******************************************************************************/
// Filename            : vsk_iic_clock_l.h
// $Revision:: 2183   $: Revision of last commit
// $Date:: 2008-05-05#$: Date of last commit
// Description         : Local header file for IIC programmable clock driver.
//******************************************************************************/

/***************************** Include Files *********************************/

#include "xparameters.h"
#include "xbasic_types.h"
#include "xiic.h"
#include "xio.h"
#include "vsk_iic_clock.h"
#include "vsk_iic_devices.h"
#include "vsk_iic.h"

//#define DEBUG		// uncomment to enable DEBUG mode
//#define NEEDS_INIT // uncomment if init is needed

/**************************** Type Definitions *******************************/

typedef Xuint8 AddressType;

/************************** Function Prototypes ******************************/

static XStatus XIicLowLevelClockDump();
static XStatus XIicLowLevelClockInit();

static Xuint8 ClockReadOneByte(AddressType Address, Xuint8 *BufferPtr);
static Xuint8 ClockReadByte(AddressType Address, Xuint8 *BufferPtr,
					  Xuint8 ByteCount);
static Xuint8 ClockWriteByte(AddressType Address, Xuint8 *BufferPtr,
					   Xuint8 ByteCount);

/************************** Constant Definitions *****************************/

/*
 * The following constants map to the XPAR parameters created in the
 * xparameters.h file. They are defined here such that a user can easily
 * change all the needed parameters in one place.
 */
#ifdef VSK_3400_BOARD
#define IIC_BASE_ADDRESS    XPAR_XPS_IIC_0_BASEADDR
#else
#define IIC_BASE_ADDRESS    0
#endif

/*
 * The Starting address in the IIC Clock on which this test is performed.
 */
#define IIC_CLOCK_TEST_START_ADDRESS   0x0

/*
 * The page size determines how much data should be written at a time.
 * The ML310/ML300 board supports a page size of 32 and 16.
 * The write function should be called with this as a maximum byte count.
 */
#define PAGE_SIZE   112

/*
 * The following constants define the number of address/data pairs in the
 * SetupBuffer and in each entry of the ConfigBuffer, respectively.  
 */
#define NUM_SETUP_ADDR 6
#define NUM_CONFIG_ADDR 12

