// DISCLAIMER OF LIABILITY
// 
// This text/file contains proprietary, confidential
// information of Xilinx, Inc., is distributed under license
// from Xilinx, Inc., and may be used, copied and/or
// disclosed only pursuant to the terms of a valid license
// agreement with Xilinx, Inc. Xilinx hereby grants you a 
// license to use this text/file solely for design, simulation, 
// implementation and creation of design files limited 
// to Xilinx devices or technologies. Use with non-Xilinx 
// devices or technologies is expressly prohibited and 
// immediately terminates your license unless covered by
// a separate agreement.
//
// Xilinx is providing this design, code, or information 
// "as-is" solely for use in developing programs and 
// solutions for Xilinx devices, with no obligation on the 
// part of Xilinx to provide support. By providing this design, 
// code, or information as one possible implementation of 
// this feature, application or standard, Xilinx is making no 
// representation that this implementation is free from any 
// claims of infringement. You are responsible for 
// obtaining any rights you may require for your implementation. 
// Xilinx expressly disclaims any warranty whatsoever with 
// respect to the adequacy of the implementation, including 
// but not limited to any warranties or representations that this
// implementation is free from claims of infringement, implied 
// warranties of merchantability or fitness for a particular 
// purpose.
//
// Xilinx products are not intended for use in life support
// appliances, devices, or systems. Use in such applications is
// expressly prohibited.
//
// Any modifications that are made to the Source Code are 
// done at the users sole risk and will be unsupported.
//
//
// Copyright (c) 2007, 2008 Xilinx, Inc. All rights reserved.
//
// This copyright and support notice must be retained as part 
// of this text at all times. 
//
//******************************************************************************/
// Filename            : display_controller.h
// $Revision:: 4166   $: Revision of last commit
// $Date:: 2008-10-13#$: Date of last commit
// Description         : Header file for display_controller Driver.
//******************************************************************************/

#ifndef DISPLAY_CONTROLLER_H
#define DISPLAY_CONTROLLER_H

/***************************** Include Files *******************************/

#include "xbasic_types.h"
#include "xstatus.h"
#include "xio.h"


/************************** Constant Definitions ***************************/

#define DISPLAY_CONTROLLER_NUM_RESOLUTIONS	15
#define DISPLAY_CONTROLLER_NUM_CONFIG_PARAMS	11

#define DISPLAY_CONTROLLER_RESET_MASK			0x80000000
#define DISPLAY_CONTROLLER_TEST_MASK			0x40000000
#define DISPLAY_CONTROLLER_INTERLACED_MASK	0x20000000

#define DISPLAY_RESOLUTION_VGA		0
#define DISPLAY_RESOLUTION_480P60	1
#define DISPLAY_RESOLUTION_576P50	2
#define DISPLAY_RESOLUTION_SVGA		3
#define DISPLAY_RESOLUTION_XGA		4
#define DISPLAY_RESOLUTION_720P50	5
#define DISPLAY_RESOLUTION_720P60	6
#define DISPLAY_RESOLUTION_1080P25	7
#define DISPLAY_RESOLUTION_1080P30	8
#define DISPLAY_RESOLUTION_1080I50	9
#define DISPLAY_RESOLUTION_1080I60	10
#define DISPLAY_RESOLUTION_NTSC		11
#define DISPLAY_RESOLUTION_PAL		12
#define DISPLAY_RESOLUTION_UTIA_ANA	13
#define DISPLAY_RESOLUTION_UTIA_DIG	14

/** @name Registers
 *
 * Register offsets for this device.
 * @{
 */
#define DISPLAY_CONTROLLER_USER_SLV_SPACE_OFFSET (0x00000000)
#define DISPLAY_CONTROLLER_CONTROL_OFFSET (DISPLAY_CONTROLLER_USER_SLV_SPACE_OFFSET + 0x00000000)
#define DISPLAY_CONTROLLER_FRAME0_BASEADDR_OFFSET (DISPLAY_CONTROLLER_USER_SLV_SPACE_OFFSET + 0x00000004)
#define DISPLAY_CONTROLLER_FRAME_ADDR_OFFSET (DISPLAY_CONTROLLER_USER_SLV_SPACE_OFFSET + 0x00000008)
#define DISPLAY_CONTROLLER_STRIDE_OFFSET (DISPLAY_CONTROLLER_USER_SLV_SPACE_OFFSET + 0x0000000C)
#define DISPLAY_CONTROLLER_HSYNC_OFFSET (DISPLAY_CONTROLLER_USER_SLV_SPACE_OFFSET + 0x00000010)
#define DISPLAY_CONTROLLER_VSYNC_POL_OFFSET (DISPLAY_CONTROLLER_USER_SLV_SPACE_OFFSET + 0x00000014)
#define DISPLAY_CONTROLLER_FRAME_SIZE_OFFSET (DISPLAY_CONTROLLER_USER_SLV_SPACE_OFFSET + 0x00000018)

/* @} */

/**
 * Software Reset Space Register Offsets
 * -- RST : software reset register
 */
#define DISPLAY_CONTROLLER_SOFT_RST_SPACE_OFFSET (0x00000100)
#define DISPLAY_CONTROLLER_RST_REG_OFFSET (DISPLAY_CONTROLLER_SOFT_RST_SPACE_OFFSET + 0x00000000)

/**
 * Software Reset Masks
 * -- SOFT_RESET : software reset
 */
#define SOFT_RESET (0x0000000A)

/**************************** Type Definitions *****************************/


/***************** Macros (Inline Functions) Definitions *******************/

/**
 *
 * Write a value to a DISPLAY_CONTROLLER register. A 32 bit write is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is written.
 *
 * @param   BaseAddress is the base address of the DISPLAY_CONTROLLER device.
 * @param   RegOffset is the register offset from the base to write to.
 * @param   Data is the data written to the register.
 *
 * @return  None.
 *
 * @note
 * C-style signature:
 * 	void DISPLAY_CONTROLLER_mWriteReg(Xuint32 BaseAddress, unsigned RegOffset, Xuint32 Data)
 *
 */
#define DISPLAY_CONTROLLER_mWriteReg(BaseAddress, RegOffset, Data) \
 	XIo_Out32((BaseAddress) + (RegOffset), (Xuint32)(Data))

/**
 *
 * Read a value from a DISPLAY_CONTROLLER register. A 32 bit read is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is read from the register. The most significant data
 * will be read as 0.
 *
 * @param   BaseAddress is the base address of the DISPLAY_CONTROLLER device.
 * @param   RegOffset is the register offset from the base to write to.
 *
 * @return  Data is the data from the register.
 *
 * @note
 * C-style signature:
 * 	Xuint32 DISPLAY_CONTROLLER_mReadReg(Xuint32 BaseAddress, unsigned RegOffset)
 *
 */
#define DISPLAY_CONTROLLER_mReadReg(BaseAddress, RegOffset) \
 	XIo_In32((BaseAddress) + (RegOffset))


/**
 *
 * Write/Read 32 bit value to/from DISPLAY_CONTROLLER user logic slave registers.
 *
 * @param   BaseAddress is the base address of the DISPLAY_CONTROLLER device.
 * @param   RegOffset is the offset from the slave register to write to or read from.
 * @param   Value is the data written to the register.
 *
 * @return  Data is the data from the user logic slave register.
 *
 * @note
 * C-style signature:
 * 	void DISPLAY_CONTROLLER_mWriteSlaveRegn(Xuint32 BaseAddress, unsigned RegOffset, Xuint32 Value)
 * 	Xuint32 DISPLAY_CONTROLLER_mReadSlaveRegn(Xuint32 BaseAddress, unsigned RegOffset)
 *
 */
#define DISPLAY_CONTROLLER_mWriteControlReg(BaseAddress, RegOffset, Value) \
 	XIo_Out32((BaseAddress) + (DISPLAY_CONTROLLER_CONTROL_OFFSET) + (RegOffset), (Xuint32)(Value))
#define DISPLAY_CONTROLLER_mWriteFrame0BaseaddrReg(BaseAddress, RegOffset, Value) \
 	XIo_Out32((BaseAddress) + (DISPLAY_CONTROLLER_FRAME0_BASEADDR_OFFSET) + (RegOffset), (Xuint32)(Value))
#define DISPLAY_CONTROLLER_mWriteFrameAddrOffsetReg(BaseAddress, RegOffset, Value) \
 	XIo_Out32((BaseAddress) + (DISPLAY_CONTROLLER_FRAME_ADDR_OFFSET) + (RegOffset), (Xuint32)(Value))
#define DISPLAY_CONTROLLER_mWriteStrideReg(BaseAddress, RegOffset, Value) \
 	XIo_Out32((BaseAddress) + (DISPLAY_CONTROLLER_STRIDE_OFFSET) + (RegOffset), (Xuint32)(Value))
#define DISPLAY_CONTROLLER_mWriteHsyncReg(BaseAddress, RegOffset, Value) \
 	XIo_Out32((BaseAddress) + (DISPLAY_CONTROLLER_HSYNC_OFFSET) + (RegOffset), (Xuint32)(Value))
#define DISPLAY_CONTROLLER_mWriteVsyncPolReg(BaseAddress, RegOffset, Value) \
 	XIo_Out32((BaseAddress) + (DISPLAY_CONTROLLER_VSYNC_POL_OFFSET) + (RegOffset), (Xuint32)(Value))
#define DISPLAY_CONTROLLER_mWriteFrameSizeReg(BaseAddress, RegOffset, Value) \
 	XIo_Out32((BaseAddress) + (DISPLAY_CONTROLLER_FRAME_SIZE_OFFSET) + (RegOffset), (Xuint32)(Value))

#define DISPLAY_CONTROLLER_mReadControlReg(BaseAddress, RegOffset) \
 	XIo_In32((BaseAddress) + (DISPLAY_CONTROLLER_CONTROL_OFFSET) + (RegOffset))
#define DISPLAY_CONTROLLER_mReadFrame0BaseaddrReg(BaseAddress, RegOffset) \
 	XIo_In32((BaseAddress) + (DISPLAY_CONTROLLER_FRAME0_BASEADDR_OFFSET) + (RegOffset))
#define DISPLAY_CONTROLLER_mReadFrameAddrOffsetReg(BaseAddress, RegOffset) \
 	XIo_In32((BaseAddress) + (DISPLAY_CONTROLLER_FRAME_ADDR_OFFSET) + (RegOffset))
#define DISPLAY_CONTROLLER_mReadStrideReg(BaseAddress, RegOffset) \
 	XIo_In32((BaseAddress) + (DISPLAY_CONTROLLER_STRIDE_OFFSET) + (RegOffset))
#define DISPLAY_CONTROLLER_mReadHsyncReg(BaseAddress, RegOffset) \
 	XIo_In32((BaseAddress) + (DISPLAY_CONTROLLER_HSYNC_OFFSET) + (RegOffset))
#define DISPLAY_CONTROLLER_mReadVsyncPolReg(BaseAddress, RegOffset) \
 	XIo_In32((BaseAddress) + (DISPLAY_CONTROLLER_VSYNC_POL_OFFSET) + (RegOffset))
#define DISPLAY_CONTROLLER_mReadFrameSizeReg(BaseAddress, RegOffset) \
 	XIo_In32((BaseAddress) + (DISPLAY_CONTROLLER_FRAME_SIZE_OFFSET) + (RegOffset))

/**
 *
 * Reset DISPLAY_CONTROLLER via software.
 *
 * @param   BaseAddress is the base address of the DISPLAY_CONTROLLER device.
 *
 * @return  None.
 *
 * @note
 * C-style signature:
 * 	void DISPLAY_CONTROLLER_mReset(Xuint32 BaseAddress)
 *
 */
#define DISPLAY_CONTROLLER_mReset(BaseAddress) \
 	XIo_Out32((BaseAddress)+(DISPLAY_CONTROLLER_RST_REG_OFFSET), SOFT_RESET)

/************************** Function Prototypes ****************************/


/**
 *
 * Run a self-test on the driver/device. Note this may be a destructive test if
 * resets of the device are performed.
 *
 * If the hardware system is not built correctly, this function may never
 * return to the caller.
 *
 * @param   baseaddr_p is the base address of the DISPLAY_CONTROLLER instance to be worked on.
 *
 * @return
 *
 *    - XST_SUCCESS   if all self-test code passed
 *    - XST_FAILURE   if any self-test code failed
 *
 * @note    Caching must be turned off for this function to work.
 * @note    Self test may fail if data memory and device are not on the same bus.
 *
 */
XStatus DISPLAY_CONTROLLER_SelfTest(void * baseaddr_p);

Xint32 DISPLAY_CONTROLLER_setInterlaced(Xuint32 baseaddr, Xint32 val);
Xint32 DISPLAY_CONTROLLER_setResolution(Xuint32 baseaddr, Xint32 res);
Xint32 DISPLAY_CONTROLLER_setBufferAddresses(Xuint32 baseaddr, Xint32 res, Xuint32 frameBaseAddr);
Xint32 DISPLAY_CONTROLLER_setReset(Xuint32 baseaddr, Xint32 reset);
Xint32 DISPLAY_CONTROLLER_setTest(Xuint32 baseaddr, Xint32 test);
Xint32 DISPLAY_CONTROLLER_configAll(Xuint32 baseaddr, Xint32 res, Xuint32 frameBaseAddr);

#endif // DISPLAY_CONTROLLER_H
