// DISCLAIMER OF LIABILITY
//
// This text/file contains proprietary, confidential
// information of Xilinx, Inc., is distributed under license
// from Xilinx, Inc., and may be used, copied and/or
// disclosed only pursuant to the terms of a valid license
// agreement with Xilinx, Inc. Xilinx hereby grants you a
// license to use this text/file solely for design, simulation,
// implementation and creation of design files limited
// to Xilinx devices or technologies. Use with non-Xilinx
// devices or technologies is expressly prohibited and
// immediately terminates your license unless covered by
// a separate agreement.
//
// Xilinx is providing this design, code, or information
// "as-is" solely for use in developing programs and
// solutions for Xilinx devices, with no obligation on the
// part of Xilinx to provide support. By providing this design,
// code, or information as one possible implementation of
// this feature, application or standard, Xilinx is making no
// representation that this implementation is free from any
// claims of infringement. You are responsible for
// obtaining any rights you may require for your implementation.
// Xilinx expressly disclaims any warranty whatsoever with
// respect to the adequacy of the implementation, including
// but not limited to any warranties or representations that this
// implementation is free from claims of infringement, implied
// warranties of merchantability or fitness for a particular
// purpose.
//
// Xilinx products are not intended for use in life support
// appliances, devices, or systems. Use in such applications is
// expressly prohibited.
//
// Any modifications that are made to the Source Code are
// done at the users sole risk and will be unsupported.
//
//
// Copyright (c) 2007 - 2009 Xilinx, Inc. All rights reserved.
//
// This copyright and support notice must be retained as part
// of this text at all times.
//
//******************************************************************************/
// Filename            : vsk_dvi_out.c
// $Revision:: 2237   $: Revision of last commit
// $Date:: 2008-05-09#$: Date of last commit
// Description         : VSK DVI output driver file
//******************************************************************************/

#include "vsk_dvi_out.h"

static Xint8 * dvi_out_mode_name[MAX_IIC_DVI_OUT_MODES]=
{
 "iic_dvi_out_<=65MHz"
,"iic_dvi_out_>65MHz"
};

static Xuint8 iic_dvi_out[MAX_IIC_DVI_OUT_MODES][MAX_IIC_DVI_OUT][2]=
{
  //--------- IIC_DVI_OUT Pixel Clock Freq <= 65MHz --------------------
 0x21,0x09
,0x1D,0x46
,0x23,0x08
,0x49,0x00
,0x33,0x08
,0x34,0x16
,0x36,0x60

  //--------- IIC_DVI_OUT Pixel Clock Freq > 65MHz --------------------
,0x21,0x09
,0x1C,0x00
,0x1D,0x46
,0x23,0x08
,0x49,0x00
,0x33,0x06
,0x34,0x26
,0x36,0xA0
};


/******************************************************************************
* This function initializes the operating mode of the dvi_out port.
*
* @param    mode contains the selected mode that the dvi_out port is to be
*           programmed.
*
* @return   None.
*
* @note     None.
*
******************************************************************************/
void vsk_dvi_out_config(Xint32 mode)
{
  Xint32 i, j;
  Xint32 locked;
  Xuint8 byte_cnt;
  Xuint8 mux_data;
  Xuint8 reg;
  Xuint8 data;
  Xuint8 r33_data;
  Xuint8 r4D_data;

  locked = 0;
#ifdef VSK_3400_BOARD
  mux_data = 0x07;
  byte_cnt = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_BASE_MUX_ADDR,
                            mux_data, &mux_data, 1);
  if (byte_cnt < 1)
  {
    print("\r\nError writing to iic_base_mux when trying to write to iic_dvi_out (CH7301C)\r\n");
    return;
  }
#endif

  for(i=0;i<MAX_IIC_DVI_OUT;i++)
  {
    byte_cnt = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_DVI_OUT_ADDR,
                             iic_dvi_out[mode][i][0], &iic_dvi_out[mode][i][1], 1);
    if (byte_cnt < 1)
    {
      xil_printf("Error writing device[0x%x][0x%x] = 0x%x\n\r", IIC_DVI_OUT_ADDR,
                 iic_dvi_out[mode][i][0], iic_dvi_out[mode][i][1]);
      return;
    }
  }

  usec_wait(50000); // sleep 50ms

  // Turn on DVIP (0x49[7:6])
  reg = 0x49;
  data = 0xC0;
  byte_cnt = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_DVI_OUT_ADDR,
                           reg, &data, 1);

  for(j=0;j<20;j++)
  {
    usec_wait(10000); // sleep 10ms

    // Reset Data path
    if (byte_cnt == 1) {
      reg = 0x48;
      data = 0x10;
      byte_cnt = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_DVI_OUT_ADDR,
                               reg, &data, 1);
    }
    if (byte_cnt == 1) {
      reg = 0x48;
      data = 0x18;
      byte_cnt = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_DVI_OUT_ADDR,
                               reg, &data, 1);
    }
    // Reset Post scale divider by toggling the divider coefficient once
      if (byte_cnt == 1) {
        reg = 0x33;
        data = 0x0C;
        byte_cnt = vsk_iic_read(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_DVI_OUT_ADDR,
                                 reg, &r33_data, 1);
      }
    if (byte_cnt == 1) {
      reg = 0x33;
      data = 0x0C;
      byte_cnt = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_DVI_OUT_ADDR,
                               reg, &data, 1);
    }
    if (byte_cnt == 1) {
      reg = 0x33;
      data = r33_data;  //0x08;
      byte_cnt = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_DVI_OUT_ADDR,
                               reg, &data, 1);
    }
    // Enable the DVI PLL lock detection function
    if (byte_cnt == 1) {
      reg = 0x35;
      data = 0x70;
      byte_cnt = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_DVI_OUT_ADDR,
                               reg, &data, 1);
    }
    if(byte_cnt < 1) {
        xil_printf("Error writing device[0x%x][0x%x] = 0x%x\n\r", IIC_DVI_OUT_ADDR,
                   reg, data);
        return;
    }

    for(i=0;i<20;i++)
    {
      usec_wait(5000); // sleep 5ms
      // Check the PLL Lock condition (0x4D[0x2]=0x1)
      byte_cnt = vsk_iic_read(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_DVI_OUT_ADDR,
                               0x4D, &r4D_data, 1);
      if(byte_cnt < 1) {
        print("\r\nError reading device[0x76][0x4D]\r\n");
        return;
      }
      if(r4D_data & 0x04) {
        print(" dvi_out pll locked\r\n");
        locked = 1;
        break;
      }

    }
    if(locked==1) { break; }
  }
  if(locked==0) { print(" dvi_out pll is not locked\r\n"); }

  // Close the Base IIC Mux
#ifdef VSK_3400_BOARD
  mux_data = 0x00;
  byte_cnt = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_BASE_MUX_ADDR,
                            mux_data, &mux_data, 1);
  if (byte_cnt < 1)
  {
    print("\r\nError writing to iic_base_mux when trying to write to iic_dvi_out (CH7301C)\r\n");
  }
  else
#endif
  {
    xil_printf(" init dvi output - mode[%x]=<%s>\n\r",mode,
               dvi_out_mode_name[mode]);
  }
}


/***********************************************************************
* This function checks to see if a video display is connected to the
* dvi_out port.
*
* @param    message specifies whether or not to print the warning message.
*             1 - print message
*             0 - do not print message
*
* @return   The result of the check
*             2 - Analog Display Present
*             1 - Digital Display Present
*             0 - Display Not Present
*
* @note     None.
*
***********************************************************************/
Xint32 vsk_dvi_out_display_present(Xint32 message)
{
  Xuint8 byte_cnt;
  Xuint8 num_bytes;
  Xuint8 data;
  Xuint8 analog;
  Xuint8 r49_data;
  Xuint8 r21_data;
  Xuint8 r20_data;
  Xuint8 mux_data;

#ifdef VSK_3400_BOARD
  mux_data = 0x07;
  num_bytes = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_BASE_MUX_ADDR,
                            mux_data, &mux_data, 1);
  if (num_bytes < 1)
  {
    print("\r\nError writing to iic_base_mux when trying to write to iic_dvi_out (CH7301C)\r\n");
  }
  else
#endif
  {

//print("Open Base_Mux for DVI_OUT\r\n");

    // Check for VGA Monitor
    byte_cnt = vsk_iic_read(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_DVI_OUT_ADDR,
                             0x49, &r49_data, 1);
  }

  if (byte_cnt == 1)
  {

    data = r49_data & 0xF0; // Force bits 3-0 to 0
    byte_cnt = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_DVI_OUT_ADDR,
                             0x49, &data, 1);
  }
  if (byte_cnt == 1)
  {

    byte_cnt = vsk_iic_read(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_DVI_OUT_ADDR,
                             0x21, &r21_data, 1);
  }
  if (byte_cnt == 1)
  {
    data = r21_data & 0xFE; // Force bits 0 to 0
    byte_cnt = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_DVI_OUT_ADDR,
                             0x21, &data, 1);
  }
  if (byte_cnt == 1)
  {
    byte_cnt = vsk_iic_read(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_DVI_OUT_ADDR,
                             0x20, &r20_data, 1);
  }
  if (byte_cnt == 1)
  {
    data = r20_data | 0x01; // Force bits 0 to 1
    byte_cnt = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_DVI_OUT_ADDR,
                             0x20, &data, 1);
  }
  if (byte_cnt == 1)
  {
    data = data & 0xFE; // Force bits 0 to 0
    byte_cnt = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_DVI_OUT_ADDR,
                             0x20, &data, 1);
  }
  if (byte_cnt == 1) // Read the status of each DAC and DVI Hot Plug Detect Bit
  {
    byte_cnt = vsk_iic_read(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_DVI_OUT_ADDR,
                             0x20, &data, 1);
  }
  if (byte_cnt ==1)
  {
    byte_cnt = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_DVI_OUT_ADDR,
                             0x49, &r49_data, 1);
  }
  if (byte_cnt ==1)
  {
    byte_cnt = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_DVI_OUT_ADDR,
                             0x21, &r21_data, 1);
  }
  if (byte_cnt ==1)
  {
    byte_cnt = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_DVI_OUT_ADDR,
                             0x20, &r20_data, 1);
  }
  else
  {
    print(" Error accessing iic_dvi_out (CH7301C)\r\n");
    return 0;
  }

  // Fix for Init Lock up when using some CRTs
  if (data & 0x2E)
  {
    byte_cnt = vsk_iic_read(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_EDID_DVI_OUT_ADDR,
                            0x00, &r20_data, 1);
  }

  // Close the Base IIC Mux
#ifdef VSK_3400_BOARD
  mux_data = 0x00;
  num_bytes = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_BASE_MUX_ADDR,
                            mux_data, &mux_data, 1);
  if (num_bytes < 1)
  {
    print("\r\nError writing to iic_base_mux when trying to write to iic_dvi_out (CH7301C)\r\n");
  }
#endif

  analog = data & 0x0E;
  if (analog > 0)
  { if (message == 1)
    {
      print(" Analog output display is present on the DVI_Out connector.\r\n");
    }
    return 2;
  }

  data = data & 0x20;
  if (data > 0)
  { if (message == 1)
    {
      print(" Digital output display is present on the DVI_Out connector.\r\n");
    }
    return 1;
  }
  else
  {
    if (message == 1)
    {
      print(" A display is not detected on the DVI_Out connector.\r\n");
      print(" Please verify that the display is properly connected.\r\n\r\n");
    }
  }
  return 0;

}
