// DISCLAIMER OF LIABILITY
// 
// This text/file contains proprietary, confidential
// information of Xilinx, Inc., is distributed under license
// from Xilinx, Inc., and may be used, copied and/or
// disclosed only pursuant to the terms of a valid license
// agreement with Xilinx, Inc. Xilinx hereby grants you a 
// license to use this text/file solely for design, simulation, 
// implementation and creation of design files limited 
// to Xilinx devices or technologies. Use with non-Xilinx 
// devices or technologies is expressly prohibited and 
// immediately terminates your license unless covered by
// a separate agreement.
//
// Xilinx is providing this design, code, or information 
// "as-is" solely for use in developing programs and 
// solutions for Xilinx devices, with no obligation on the 
// part of Xilinx to provide support. By providing this design, 
// code, or information as one possible implementation of 
// this feature, application or standard, Xilinx is making no 
// representation that this implementation is free from any 
// claims of infringement. You are responsible for 
// obtaining any rights you may require for your implementation. 
// Xilinx expressly disclaims any warranty whatsoever with 
// respect to the adequacy of the implementation, including 
// but not limited to any warranties or representations that this
// implementation is free from claims of infringement, implied 
// warranties of merchantability or fitness for a particular 
// purpose.
//
// Xilinx products are not intended for use in life support
// appliances, devices, or systems. Use in such applications is
// expressly prohibited.
//
// Any modifications that are made to the Source Code are 
// done at the users sole risk and will be unsupported.
//
//
// Copyright (c) 2007, 2008 Xilinx, Inc. All rights reserved.
//
// This copyright and support notice must be retained as part 
// of this text at all times. 
//
//******************************************************************************/
// Filename            : vsk_iic_diag.c
// $Revision:: 3121   $: Revision of last commit
// $Date:: 2008-07-29#$: Date of last commit
// Description         : VSK IIC Diagnostics Menu
//******************************************************************************/

#include "vsk_iic_diag.h"


static Xint8 * iic_device[MAX_IIC_DEV]=
{
  "iic_dvi_out (CH7301C)"  
 ,"iic_dvi_in (AD9984A)"
 ,"iic_io_exp (PCA9555)"
 ,"iic_sd_in (ADV7180)"
 ,"iic_sd_out (ADV7179)"
 ,"iic_fmc_mux (PCA9546)"
 ,"iic_camera_1 (MT9V022)"
 ,"iic_camera_2 (MT9V022)"
 ,"iic_edid_dvi_in" 
 ,"iic_base_mux (PCA9544)"
 ,"iic_edid_dvi_out"
 ,"iic_fmc_power"
} ;

static Xint8 iic_addr[MAX_IIC_DEV]=
{
  IIC_DVI_OUT_ADDR        // 0x76 (0xEC/2)
 ,IIC_DVI_IN_ADDR         // 0x4D (0x9A/2)
 ,IIC_IO_EXP_ADDR         // 0x20 (0x40/2)
 ,IIC_SD_IN_ADDR          // 0x21 (0x42/2)
 ,IIC_SD_OUT_ADDR         // 0x2A (0x54/2)
 ,IIC_FMC_MUX_ADDR        // 0x74 (0xE8/2)
 ,IIC_EPIX_CAMERA_ADDR    // 0x58 (0xB0/2)
 ,IIC_EPIX_CAMERA_ADDR    // 0x58 (0xB0/2)
 ,IIC_EDID_DVI_IN_ADDR    // 0x50 (0xA0/2)
 ,IIC_BASE_MUX_ADDR       // 0x72 (0xE4/2)
 ,IIC_EDID_DVI_OUT_ADDR   // 0x50 (0xA0/2)
 ,IIC_FMC_POWER_ADDR      // 0x28 (0x50/2)
} ;


void vsk_iic_diag_help(void);
Xuint8 get_camera_addr(Xint32 camera);


/*****************************************************************************
* This function creates a menu structure that allows the user the ability to 
*  read and write the IIC registers of the chips in the VSK in real-time.
*
* @param    None.
*
* @return   Returns '0' when it exits successfully.
*
* @note     None.
*
******************************************************************************/
Xint32 vsk_iic_diag_main (void)
{ 
  Xint32 inchar;
  Xint32 i;
  static Xint32  data=0, addr=0, device=0;
  Xuint8 xdata[2];
  Xint8 mux_data;
  Xuint8  num_bytes;

  Xuint8 camera1_addr = 0xFF; // Not a valid address. 
  Xuint8 camera2_addr = 0xFF; // Means the camera has not been checked for.

  xdata[1] = 0; xdata[0] = 0;

  vsk_iic_diag_help();

  while (1)
  {
    print(">");

    inchar = inbyte();

    if(inchar != 0x1b) { xil_printf("%c\n\r",inchar); }
    else               { print("esc\n\r"); }

    switch (inchar)
    {
      case '?' : 
      {
        vsk_iic_diag_help(); 
        break;
      }          

      case 0x1b :
      {
        xil_printf("- exit menu -\n\n\r");
        return(0);
        break;
      }

      // device 
      case '=' :
      {
        device++; 
        if(device>MAX_IIC_DEV-1){ device=MAX_IIC_DEV-1; }
        xil_printf("device[%x]:IIC(0x%x):<%s> regs\n \r", device ,iic_addr[device],iic_device[device]);
        break;
      }
        
      case '-' :
      {
        device--; 
        if(device<0) { device=0; }
        xil_printf("device[%x]:IIC(0x%x):<%s> regs,  \n \r", device ,iic_addr[device],iic_device[device]);
        break;
      }

      // addr
      case ']' :
      {
        addr++; 
        if(addr>0xff){ addr=0xff; }
        xil_printf("IIC_reg[%x][%x] = <%s>\n\r",device, addr, iic_device[device]);
        break;
      }

      case '[' :
      {
        addr--; 
        if(addr<0){ addr=0; }
        xil_printf("IIC_reg[%x][%x] = <%s>\n\r",device, addr, iic_device[device]);      
        break;
      }
      
      case '}' :
      {
        addr+=0x10; 
        if(addr>0xff){ addr=0xff; }
        xil_printf("IIC_reg[%x][%x] = <%s>\n\r",device, addr, iic_device[device]);
        break;
      }

      case '{' :
      {
        addr-=0x10; 
        if(addr<0){ addr=0; }
        xil_printf("IIC_reg[%x][%x] = <%s>\n\r",device, addr, iic_device[device]);      
        break;
      }

      // data
      case '\'' :
      {
        data++;  
        xdata[1] = (Xint8)(data & 0xFF);
        xdata[0] = (Xint8)((data>>8) & 0xFF);        
        xil_printf("data=0x%x\n\r",data);
//        xil_printf("xdata=0x%x %x\n\r",xdata[0],xdata[1]);
        break;
      }
        
      case ';' :
      {
        data--;
        xdata[1] = (Xint8)(data & 0xFF);
        xdata[0] = (Xint8)((data>>8) & 0xFF);        
        xil_printf("data=0x%x\n\r",data);
//        xil_printf("xdata=0x%x %x\n\r",xdata[0],xdata[1]);
        break;
      }

      case '\"' :
      {
        data=data+0x100;  
        xdata[1] = (Xint8)(data & 0xFF);
        xdata[0] = (Xint8)((data>>8) & 0xFF);        
        xil_printf("data=0x%x\n\r",data);
//        xil_printf("xdata=0x%x %x\n\r",xdata[0],xdata[1]);
        break;
      }

      case ':' :
      {
        data=data-0x100;
        xdata[1] = (Xint8)(data & 0xFF);
        xdata[0] = (Xint8)((data>>8) & 0xFF);        
        xil_printf("data=0x%x\n\r",data);
//        xil_printf("xdata=0x%x %x\n\r",xdata[0],xdata[1]);
        break;
      }

      // write register
      case '.' : 
      {
        num_bytes = 1;
        if(device ==IIC_DVI_OUT || device ==IIC_EDID_DVI_OUT)
        {
          // Open the Base IIC Mux
          mux_data = 0x07;
//          num_bytes = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_BASE_MUX_ADDR, 
//                                    mux_data, &mux_data, 1);
//          if (num_bytes < 1) 
//          { 
//            print("\r\nError writing to iic_base_mux when trying to write to %s\r\n",iic_device[device]); 
//          }
//          else 
//          {                                      
            num_bytes = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, iic_addr[device], 
                                      addr, &xdata[1], 1); 
//          }
        }
//        else if(device ==IIC_FMC_POWER) 
//        { 
//          // Open the Base IIC Mux
//          mux_data = 0x06;
//          num_bytes = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_BASE_MUX_ADDR, 
//                                    mux_data, &mux_data, 1);
//          if (num_bytes < 1) 
//          { 
//            print("\r\nError writing to iic_base_mux when trying to write to %s\r\n",iic_device[device]); 
//          }
//          else 
//          {                                      
//            num_bytes = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, iic_addr[device], 
//                                      addr, &xdata[1], 1);
//          }
//        }
//        else if(device ==IIC_BASE_MUX)         
//        { 
//          num_bytes = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, iic_addr[device], 
//                                   addr, &xdata[1], 1); 
//        }
//        else if(device ==IIC_CAMERA_1) 
//        { 
//          if (camera1_addr == 0xFF || camera1_addr == 0x00)
//          {
//            camera1_addr = get_camera_addr(1);
//          }
//          if (camera1_addr != 0)
//          {
//            mux_data = 0x02;
//            num_bytes = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, IIC_FMC_MUX_ADDR, 
//                                      mux_data, &mux_data, 1);
//            if (num_bytes < 1) 
//            { 
//              print("\r\nError writing to iic_fmc_mux when trying to write iic_camera_1\r\n"); 
//            }
//            else 
//            {                                      
//              num_bytes = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, camera1_addr, 
//                                        addr, xdata, 2);
//            }
//          }
//          else { print("Camera #1 is not present.\r\n"); break; }        
//        }
//        else if(device ==IIC_CAMERA_2) 
//        { 
//          if (camera2_addr == 0xFF || camera2_addr == 0x00)
//          {
//            camera2_addr = get_camera_addr(2);
//          }
//          if (camera2_addr != 0)
//          {
//            mux_data = 0x04;
//            num_bytes = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, IIC_FMC_MUX_ADDR, 
//                                      mux_data, &mux_data, 1);
//            if (num_bytes < 1) 
//            { 
//              print("\r\nError writing to iic_fmc_mux when trying to write iic_camera_2\r\n"); 
//            }
//            else 
//            {                                      
//              num_bytes = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, camera2_addr, 
//                                        addr, xdata, 2);
//            }
//          }
//          else  { print("Camera #2 is not present.\r\n"); break; }
//        }
//        else if(device ==IIC_EDID_DVI_IN) 
//        { 
//          mux_data = 0x01;
//          num_bytes = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, IIC_FMC_MUX_ADDR, 
//                                    mux_data, &mux_data, 1);
//          if (num_bytes < 1) 
//          { 
//            print("\r\nError writing to iic_fmc_mux when trying to write iic_edid_dvi_in\r\n"); 
//          }
//          else 
//          {                                      
//            num_bytes = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, iic_addr[device], 
//                                      addr, &xdata[1], 1);
//          }                                    
//        }
//        else 
//        { 
//          num_bytes = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, iic_addr[device], 
//                                    addr, &xdata[1], 1); 
//        }

        if (num_bytes < 1) 
        {
          xil_printf("\r\nError writing to device<%s> = %d \r\n\r\n",iic_device[device],num_bytes);
        }
//        else
//        {
//          if (device ==IIC_CAMERA_1 || device ==IIC_CAMERA_2)
//          {
//            xil_printf("write IIC_reg[0x%x][0x%x]=0x%x (%d) \n\r",iic_addr[device],addr,data,data);
//          }
          else
          {
            xil_printf("write IIC_reg[0x%x][0x%x]=0x%x (%d) \n\r",iic_addr[device],addr,(data&0xFF),(data&0xFF));
          }                  
//        }

//        if (device ==IIC_CAMERA_1 || device ==IIC_CAMERA_2 || device ==IIC_EDID_DVI_IN)
//        {
//          // FMC IIC Mux Disable
//          mux_data = 0x08;
//          num_bytes = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, IIC_FMC_MUX_ADDR, 
//                                    mux_data, &mux_data, 1);
//          if (num_bytes < 1) 
//          { 
//            print("\r\nError writing to iic_fmc_mux\r\n"); 
//          }
//        }
        
//        if(device ==IIC_DVI_OUT || device ==IIC_EDID_DVI_OUT || device ==IIC_FMC_POWER)
//        {
//          // Close the Base IIC Mux
//          mux_data = 0x00;
//          num_bytes = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_BASE_MUX_ADDR, 
//                                    mux_data, &mux_data, 1);
//          if (num_bytes < 1) 
//          { 
//            print("\r\nError writing to iic_base_mux when trying to write to %s\r\n",iic_device[device]); 
//          }
//        }
        
        break;
      }
     
      // read register
      case ',' : 
      {
        num_bytes = 1;
        if(device ==IIC_DVI_OUT || device ==IIC_EDID_DVI_OUT)
        {
//          mux_data = 0x07;
//          num_bytes = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_BASE_MUX_ADDR, 
//                                    mux_data, &mux_data, 1);
//          if (num_bytes < 1) 
//          { 
//            print("\r\nError writing to iic_base_mux when trying to write to %s\r\n",iic_device[device]); 
//          }
//          else 
//          {                                      
            num_bytes = vsk_iic_read(VIDEO_OUT_IIC_BASE_ADDRESS, iic_addr[device], 
                                      addr, &xdata[1], 1); 
            data = (int)xdata[1]; 
//          }
        }
//        else if(device ==IIC_FMC_POWER)         
//        { 
//          mux_data = 0x06;
//          num_bytes = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_BASE_MUX_ADDR, 
//                                    mux_data, &mux_data, 1);
//          if (num_bytes < 1) 
//          { 
//            print("\r\nError writing to iic_base_mux when trying to write to %s\r\n",iic_device[device]); 
//          }
//          else 
//          {                                      
//            num_bytes = vsk_iic_read(VIDEO_OUT_IIC_BASE_ADDRESS, iic_addr[device], 
//                                     addr, &xdata[1], 1); 
//            data = (int)xdata[1];
//          }
//        }
//        else if(device ==IIC_BASE_MUX)         
//        { 
//          num_bytes = vsk_iic_read(VIDEO_OUT_IIC_BASE_ADDRESS, iic_addr[device], 
//                                   addr, &xdata[1], 1); 
//          data = (int)xdata[1];
//        }
//        else if(device ==IIC_CAMERA_1) 
//        { 
//          if (camera1_addr == 0xFF || camera1_addr == 0x00)
//          {
//            camera1_addr = get_camera_addr(1);
//          }
//          if (camera1_addr != 0)
//          {
//            mux_data = 0x02;
//            num_bytes = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, IIC_FMC_MUX_ADDR, 
//                                      mux_data, &mux_data, 1);
//            if (num_bytes < 1) 
//            { 
//              print("\r\nError writing to iic_fmc_mux when trying to read camera_1\r\n"); 
//            }
//            else 
//            {                                      
//              num_bytes = vsk_iic_read(FMC_CARD_IIC_BASE_ADDRESS, camera1_addr, 
//                                       addr, xdata, 2);
//              data = (((int)xdata[0])<<8) + (int)xdata[1]; 
//            }
//          }            
//          else  { print("Camera #1 is not present.\r\n"); break; }
//        }
//        else if(device ==IIC_CAMERA_2) 
//        { 
//          if (camera2_addr == 0xFF || camera2_addr == 0x00)
//          {
//            camera2_addr = get_camera_addr(2);
//          }
//          if (camera2_addr != 0)
//          {
//            mux_data = 0x04;
//            num_bytes = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, IIC_FMC_MUX_ADDR, 
//                                      mux_data, &mux_data, 1);
//            if (num_bytes < 1) 
//            { 
//              print("\r\nError writing to iic_fmc_mux when trying to read camera_2\r\n"); 
//            }
//            else 
//            {                                      
//              num_bytes = vsk_iic_read(FMC_CARD_IIC_BASE_ADDRESS, camera2_addr, 
//                                       addr, xdata, 2);
//              data = (((int)xdata[0])<<8) + (int)xdata[1]; 
//            }
//          }            
//          else  { print("Camera #2 is not present.\r\n"); break; }
//        }
//        else if(device ==IIC_EDID_DVI_IN) 
//        { 
//          mux_data = 0x01;
//          num_bytes = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, IIC_FMC_MUX_ADDR, 
//                                    mux_data, &mux_data, 1);
//          if (num_bytes < 1) 
//          { 
//            print("\r\nError writing to iic_fmc_mux when trying to read device<iic_edid_dvi_in>\r\n"); 
//          }
//          else 
//          {                                      
//            num_bytes = vsk_iic_read(FMC_CARD_IIC_BASE_ADDRESS, iic_addr[device], 
//                                     addr, &xdata[1], 1);
//            data = (int)xdata[1]; 
//          }                                    
//        }
//        else 
//        { 
//          num_bytes = vsk_iic_read(FMC_CARD_IIC_BASE_ADDRESS, iic_addr[device], 
//                                   addr, &xdata[1], 1); 
//          data = (int)xdata[1]; 
//        }

        if (num_bytes < 1) 
        {
          xil_printf("\r\nError reading from device<%s> = %d \r\n\r\n",iic_device[device],num_bytes);
        }
//        else
//        {        
//          if (device ==IIC_CAMERA_1 || device ==IIC_CAMERA_2)
//          {
//            xil_printf("read IIC_reg[0x%x][0x%x]=0x%x (%d) \n\r",iic_addr[device],addr,data,data);
//          }
          else
          {
            xil_printf("read IIC_reg[0x%x][0x%x]=0x%x (%d) \n\r",iic_addr[device],addr,(data&0xFF),(data&0xFF));
          }                  
//        }
        
//        if (device ==IIC_CAMERA_1 || device ==IIC_CAMERA_2 || device ==IIC_EDID_DVI_IN)
//        {
//          // FMC IIC Mux Disable
//          mux_data = 0x08;
//          num_bytes = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, IIC_FMC_MUX_ADDR, 
//                                    mux_data, &mux_data, 1);
//          if (num_bytes < 1) 
//          { 
//            print("\r\nError writing to iic_fmc_mux\r\n"); 
//          }
//        }
        
//        if(device ==IIC_DVI_OUT || device ==IIC_EDID_DVI_OUT || device ==IIC_FMC_POWER)
//        {
//          // Close the Base IIC Mux
//          mux_data = 0x00;
//          num_bytes = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_BASE_MUX_ADDR, 
//                                    mux_data, &mux_data, 1);
//          if (num_bytes < 1) 
//          { 
//            print("\r\nError writing to iic_base_mux when trying to write to %s\r\n",iic_device[device]); 
//          }
//        }
        
        break;
      }
      
      // Dump All Device Registers
      case 'd' : 
      {
        num_bytes = 1;
        xil_printf("display device memory: device[%x]=<%s>\r\n\r\n",device, 
                    iic_device[device]); 

        // Setup the BASE IIC MUX if needed
        if(device ==IIC_DVI_OUT || device ==IIC_EDID_DVI_OUT)
        {
//          mux_data = 0x07;
//          num_bytes = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_BASE_MUX_ADDR, 
//                                    mux_data, &mux_data, 1);
//          if (num_bytes < 1) 
//          { 
//            xil_printf("\r\nError writing to iic_base_mux when trying to write to %s\r\n",iic_device[device]); 
//            break;            
//          }
        }
//        else if(device ==IIC_FMC_POWER)
//        {
//          mux_data = 0x06;
//          num_bytes = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_BASE_MUX_ADDR, 
//                                    mux_data, &mux_data, 1);
//          if (num_bytes < 1) 
//          { 
//            print("\r\nError writing to iic_base_mux when trying to write to iic_fmc_power\r\n",iic_device[device]); 
//            break;            
//          }
//        }
//        // Setup the FMC IIC MUX if needed
//        if(device ==IIC_EDID_DVI_IN) 
//        {  
//          mux_data = 0x01;
//          num_bytes = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, iic_addr[IIC_FMC_MUX], 
//                                    mux_data, &mux_data, 1);
//          if (num_bytes < 1) 
//          { 
//            print("\r\nError writing to iic_fmc_mux when trying to read device<iic_edid_dvi_in>\r\n");
//            break;            
//          }
//        }
//        else if(device ==IIC_CAMERA_1) 
//        {
//          if (camera1_addr == 0xFF || camera1_addr == 0x00)
//          {
//            camera1_addr = get_camera_addr(1);
//          }
//          if (camera1_addr != 0)
//          {
//            mux_data = 0x02;
//            num_bytes = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, IIC_FMC_MUX_ADDR, 
//                                      mux_data, &mux_data, 1);
//            if (num_bytes < 1) 
//            { 
//              print("\r\nError writing to iic_fmc_mux when trying to read camera_1\r\n"); 
//            }
//          }
//          else  { print("Camera #1 is not present.\r\n"); break; }          
//        }
//        else if(device ==IIC_CAMERA_2) 
//        {
//          if (camera2_addr == 0xFF || camera2_addr == 0x00)
//          {
//            camera2_addr = get_camera_addr(2);
//          }
//          if (camera2_addr != 0)
//          {
//            mux_data = 0x04;
//            num_bytes = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, IIC_FMC_MUX_ADDR, 
//                                      mux_data, &mux_data, 1);
//            if (num_bytes < 1) 
//            { 
//              print("\r\nError writing to iic_fmc_mux when trying to read camera_2\r\n"); 
//            }
//          }
//          else  { print("Camera #2 is not present.\r\n"); break; }          
//        }

        // Read registers 0x00 - 0xFF
        for(i=0; i<=0xFF; i++) 
        {
          if(device ==IIC_DVI_OUT || device ==IIC_EDID_DVI_OUT || device ==IIC_FMC_POWER || device ==IIC_BASE_MUX)          
          { 
            num_bytes = vsk_iic_read(VIDEO_OUT_IIC_BASE_ADDRESS, iic_addr[device], i, &xdata[1], 1); 
            data = (int)xdata[1]; 
          }
//          else if(device ==IIC_CAMERA_1 ) 
//          { 
//            num_bytes = vsk_iic_read(FMC_CARD_IIC_BASE_ADDRESS, camera1_addr, 
//                                     i, xdata, 2);
//            data = (((int)xdata[0])<<8) + (int)xdata[1]; 
//            if(num_bytes < 2) { num_bytes = 0; } 
//          }                                    
//          else if(device ==IIC_CAMERA_2) 
//          { 
//            num_bytes = vsk_iic_read(FMC_CARD_IIC_BASE_ADDRESS, camera2_addr, 
//                                     i, xdata, 2);
//            data = (((int)xdata[0])<<8) + (int)xdata[1]; 
//            if(num_bytes < 2) { num_bytes = 0; } 
//          }                                    
//          else 
//          { 
//            num_bytes = vsk_iic_read(FMC_CARD_IIC_BASE_ADDRESS, iic_addr[device], 
//                                     i, &xdata[1], 1); 
//            data = (int)xdata[1]; 
//          }
          
          if (num_bytes < 1) 
          {
            xil_printf("\r\nError reading from device<%s> \r\n\r\n",iic_device[device]);
            break;
          }
          else
          {        
            xil_printf("read IIC_reg[%x][%x]=0x%x (%d) \n\r",device,i,data,data);  
          }          
        }
        
//        if (device ==IIC_CAMERA_1 || device ==IIC_CAMERA_2 || device ==IIC_EDID_DVI_IN)
//        {
//          // FMC IIC Mux Disable
//          mux_data = 0x08;
//          num_bytes = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, IIC_FMC_MUX_ADDR, 
//                                    mux_data, &mux_data, 1);
//          if (num_bytes < 1) 
//          { 
//            print("\r\nError writing to iic_fmc_mux\r\n"); 
//          }
//        }
        
//        if(device ==IIC_DVI_OUT || device ==IIC_EDID_DVI_OUT || device ==IIC_FMC_POWER)
//        {
//          // Close the Base IIC Mux
//          mux_data = 0x00;
//          num_bytes = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_BASE_MUX_ADDR, 
//                                    mux_data, &mux_data, 1);
//          if (num_bytes < 1) 
//          { 
//            print("\r\nError writing to iic_base_mux when trying to write to %s\r\n",iic_device[device]); 
//          }
//        }

        break;
      }      
    }
  }
}


void vsk_iic_diag_help(void)
{
  print("\n\r");
  print("----------------------------\n\r");
  print("--  IIC Diagnostics Menu  --\r\n");
  print("----------------------------\n\r");
  print("key(s)  description               \n\r");
  print("----------------------------\n\r");
  print(" - =  select device   \n\r");
  print(" [ ]  select register \n\r");
  print(" ; \'  data -/+ 0x001 \n\r");
  print(" : \"  data -/+ 0x100 \n\r");
  print("  .   write device[register]=data \n\r");
  print("  ,   read  device[register]\n\r");
  print("\n\r");
  print("  d   display device regs  \n\r");
  print("\n\r");
  print(" esc quit \n\r");
  print("  ?   help \n\r");
  print("----------------------------\n\r");
}


/*********************************************************************
**
* This function gets the addr of the specified camera.
*
* @param    camera port to check.
*            1 = Camera 1
*            2 = Camera 2
*
* @return   I2C address of the camera
*            I2C address of the camera = Source Present
*            0 = Source Not Present
*
* @note None.
*
**********************************************************************
*/
Xuint8 get_camera_addr(Xint32 camera)
{
/*  Xuint8 mux_data;
  Xuint8  num_bytes;
  Xuint8  mux_bytes;
  Xuint8 xdata[2];
  Xuint8 camera_addr;

  xdata[1] = 0x00;
  xdata[0] = 0x00;
  num_bytes = 0;
  mux_bytes = 0;

  // Select which camera to look for
  if (camera == 2) { mux_data = 0x04; }
  else             { mux_data = 0x02; }
  
  num_bytes = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, IIC_FMC_MUX_ADDR, 
                            mux_data, &mux_data, 1);
  if (num_bytes < 1) 
  { 
    xil_printf("\r\nError writing to iic_fmc_mux when trying to access camera #%d\r\n",camera); 
  }
  else 
  { 
    // Read the MT9V022 Chip Version Register of the EPIX camera 
    num_bytes = vsk_iic_read(FMC_CARD_IIC_BASE_ADDRESS, IIC_EPIX_CAMERA_ADDR, 
                             0x00, xdata, 2);
    camera_addr = IIC_EPIX_CAMERA_ADDR;
  }

  if (num_bytes < 2)
  {
    // Read the MT9V022 Chip Version Register of the Irvine Sensors camera 
    num_bytes = vsk_iic_read(FMC_CARD_IIC_BASE_ADDRESS, IIC_IRVINE_CAMERA_ADDR, 
                             0x00, xdata, 2);
    camera_addr = IIC_IRVINE_CAMERA_ADDR;
  }

  // FMC IIC Mux Disable
  mux_data = 0x08;
  mux_bytes = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, IIC_FMC_MUX_ADDR, 
                            mux_data, &mux_data, 1);
  if (mux_bytes < 1) 
  { 
    xil_printf("\r\nError writing to iic_fmc_mux when trying to access camera #%d\r\n",camera); 
  }
  
  if ((num_bytes < 2) | (xdata[0] != 0x13))
  {
    return 0;
  }*/
  
  return 0;
}

