// DISCLAIMER OF LIABILITY
// 
// This text/file contains proprietary, confidential
// information of Xilinx, Inc., is distributed under license
// from Xilinx, Inc., and may be used, copied and/or
// disclosed only pursuant to the terms of a valid license
// agreement with Xilinx, Inc. Xilinx hereby grants you a 
// license to use this text/file solely for design, simulation, 
// implementation and creation of design files limited 
// to Xilinx devices or technologies. Use with non-Xilinx 
// devices or technologies is expressly prohibited and 
// immediately terminates your license unless covered by
// a separate agreement.
//
// Xilinx is providing this design, code, or information 
// "as-is" solely for use in developing programs and 
// solutions for Xilinx devices, with no obligation on the 
// part of Xilinx to provide support. By providing this design, 
// code, or information as one possible implementation of 
// this feature, application or standard, Xilinx is making no 
// representation that this implementation is free from any 
// claims of infringement. You are responsible for 
// obtaining any rights you may require for your implementation. 
// Xilinx expressly disclaims any warranty whatsoever with 
// respect to the adequacy of the implementation, including 
// but not limited to any warranties or representations that this
// implementation is free from claims of infringement, implied 
// warranties of merchantability or fitness for a particular 
// purpose.
//
// Xilinx products are not intended for use in life support
// appliances, devices, or systems. Use in such applications is
// expressly prohibited.
//
// Any modifications that are made to the Source Code are 
// done at the users sole risk and will be unsupported.
//
//
// Copyright (c) 2007, 2008 Xilinx, Inc. All rights reserved.
//
// This copyright and support notice must be retained as part 
// of this text at all times. 
//
//******************************************************************************/
// Filename            : vsk_fmc_video.c
// $Revision:: 3518   $: Revision of last commit
// $Date:: 2008-09-04#$: Date of last commit
// Description         : VSK FMC-Video card driver file
//******************************************************************************/

#include "vsk_fmc_video.h"

/*****************************************************************************
* This function initializes the FMC Video Power Control IC.
*
* @param    None.
*
* @return   Returns '1' on success, '0' on failure.
*
* @note     None.
*
******************************************************************************/
Xuint8 vsk_fmc_power_init(void)
{
  Xuint8 num_bytes;
  Xuint8 mux_data;
  Xuint8 data;

    // Rev D                           
  mux_data = 0x06;
  num_bytes = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_BASE_MUX_ADDR, 
                            mux_data, &mux_data, 1);
  if (num_bytes < 1) 
  { 
    print("\r\nError writing to iic_base_mux when trying to write to iic_fmc_power\r\n"); 
  }
  else 
  {                                      
    data = 0x80;
    num_bytes = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_FMC_POWER_ADDR, 
                             0x08, &data, 1);
  }

  if (num_bytes == 1) 
  { 
    data = 0x00;  
    num_bytes = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_FMC_POWER_ADDR, 
                             0x01, &data, 1);
  }
  
  if (num_bytes == 1) 
  { 
    data = 0x00;  
    num_bytes = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_FMC_POWER_ADDR, 
                             0x00, &data, 1);
  }
  
  if (num_bytes < 1) { print("Error writing to FMC Power Control IIC.\r\n"); }

  if (num_bytes ==1)
  {
    // Close the Base IIC Mux
    mux_data = 0x00;
    num_bytes = vsk_iic_write(VIDEO_OUT_IIC_BASE_ADDRESS, IIC_BASE_MUX_ADDR, 
                              mux_data, &mux_data, 1);
    if (num_bytes < 1) 
    { 
      print("\r\nError writing to iic_base_mux when trying to write to iic_fmc_power\r\n"); 
    }
  }  

//  XENV_USLEEP(50000); // sleep 50ms to allow voltage to come up
  usec_wait(50000); // sleep 50ms to allow voltage to come up
  return num_bytes;
}


/*****************************************************************************
* This function resets the everything on the FMC Video card.
*
* @param    None.
*
* @return   Returns '1' on success, '0' on failure.
*
* @note     None.
*
******************************************************************************/
Xuint8  vsk_fmc_reset(void)
{
  Xuint8  byte_cnt;
  Xuint8 data;
  Xuint8 device;

  //Initialize the I/O Exp ports
  device = IIC_IO_EXP_ADDR;
  data = 0x4A;
//  byte_cnt = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, device, 
//                           0x06, &data, 1);
  if (byte_cnt == 1) 
  {
    data = 0x45;  
//    byte_cnt = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, device, 
//                             0x07, &data, 1);
  }
  // Power down and disable everything on the I/O Exp
  if (byte_cnt == 1) 
  {  
    data = 0x00;
//    byte_cnt = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, device, 
//                             0x02, &data, 1);
  }
  if (byte_cnt == 1) 
  {  
    data = 0x80;
//    byte_cnt = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, device, 
//                             0x03, &data, 1);
  }
  // Release Reset signal on I/O Exp
  if (byte_cnt == 1) 
  {  
    device = IIC_IO_EXP_ADDR;
    data = 0x01;
//    byte_cnt = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, device, 
//                             0x02, &data, 1);
  }
  // Set FMC I2C Mux to init state
  if (byte_cnt == 1) 
  {  
    device = IIC_FMC_MUX_ADDR;
    data = 0x08;
//    byte_cnt = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, device, 
//                             data, &data, 1);
  }
  // Disable SDTV IN (ADV7180) (shared data bus)
  if (byte_cnt == 1) 
  {  
    device = IIC_SD_IN_ADDR;
    data = 0x4C;
//    byte_cnt = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, device, 
//                             0x03, &data, 1);
  }
  // Disable Analog DVI IN (AD9984A) (shared data bus)
  if (byte_cnt == 1) 
  {  
    device = IIC_DVI_IN_ADDR;
    data = 0x25;
//    byte_cnt = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, device, 
//                             0x20, &data, 1);
  }
  // Turn on FMC-Video Status LED
  if (byte_cnt == 1) 
  {  
    device = IIC_IO_EXP_ADDR;
    data = 0x00; 
//    byte_cnt = vsk_iic_write(FMC_CARD_IIC_BASE_ADDRESS, device, 
//                             0x03, &data, 1);
  }

  if (byte_cnt < 1) 
  { 
    xil_printf("Error writing to IIC_device<0x%x> during FMC Reset.\r\n",
               device);
  }

  return byte_cnt;
}
