// DISCLAIMER OF LIABILITY
//
// This text/file contains proprietary, confidential
// information of Xilinx, Inc., is distributed under license
// from Xilinx, Inc., and may be used, copied and/or
// disclosed only pursuant to the terms of a valid license
// agreement with Xilinx, Inc. Xilinx hereby grants you a
// license to use this text/file solely for design, simulation,
// implementation and creation of design files limited
// to Xilinx devices or technologies. Use with non-Xilinx
// devices or technologies is expressly prohibited and
// immediately terminates your license unless covered by
// a separate agreement.
//
// Xilinx is providing this design, code, or information
// "as-is" solely for use in developing programs and
// solutions for Xilinx devices, with no obligation on the
// part of Xilinx to provide support. By providing this design,
// code, or information as one possible implementation of
// this feature, application or standard, Xilinx is making no
// representation that this implementation is free from any
// claims of infringement. You are responsible for
// obtaining any rights you may require for your implementation.
// Xilinx expressly disclaims any warranty whatsoever with
// respect to the adequacy of the implementation, including
// but not limited to any warranties or representations that this
// implementation is free from claims of infringement, implied
// warranties of merchantability or fitness for a particular
// purpose.
//
// Xilinx products are not intended for use in life support
// appliances, devices, or systems. Use in such applications is
// expressly prohibited.
//
// Any modifications that are made to the Source Code are
// done at the users sole risk and will be unsupported.
//
//
// Copyright (c) 2007, 2008 Xilinx, Inc. All rights reserved.
//
// This copyright and support notice must be retained as part
// of this text at all times.
//
//******************************************************************************/
// Filename            : vsk_iic.h
// $Revision:: 2238   $: Revision of last commit
// $Date:: 2008-05-09#$: Date of last commit
// Description         : VSK IIC controller driver header file
//******************************************************************************/

#ifndef __vsk_iic__
#define __vsk_iic__

/***************************** Include Files *********************************/
//#include "xutil.h"
#include "xparameters.h"
#include "xbasic_types.h"
#include "xiic.h"
#include "xio.h"

/*
 * The following constants map to the XPAR parameters created in the
 * xparameters.h file. They are defined here such that a user can easily
 * change all the needed parameters in one place.
 */
#define VIDEO_OUT_IIC_BASE_ADDRESS   XPAR_IIC_DVI_BASEADDR
#define FMC_CARD_IIC_BASE_ADDRESS    XPAR_XPS_IIC_1_BASEADDR

/*
 * The page size determines how much data should be written at a time.
 * The write function should be called with this as a maximum byte count.
 */
#define IIC_PAGE_SIZE   2

/**************************** Type Definitions *******************************/

/************************** Function Prototypes ******************************/

/******************************************************************************
* This function initializes the IIC cores in the VSK.
*
* @param    None.
*
* @return   None.
*
* @note     None.
*
******************************************************************************/
void vsk_iic_init(void);


/******************************************************************************
* This function reads a number of bytes from an IIC chip into a
* specified buffer.
*
* @param    CoreAddress contains the address of the IIC core.
* @param    ChipAddress contains the address of the IIC core.
* @param    RegAddress contains the address of the register to write to.
* @param    BufferPtr contains the address of the data buffer to be filled.
* @param    ByteCount contains the number of bytes in the buffer to be read.
*           This value is constrained by the page size of the device such
*           that up to 64K may be read in one call.
*
* @return   The number of bytes read. A value less than the specified input
*           value indicates an error.
*
* @note     None.
*
******************************************************************************/
Xuint8  vsk_iic_read(Xuint32 CoreAddress, Xuint8 ChipAddress, Xuint8 RegAddress,
                 Xuint8 *BufferPtr, Xuint8 ByteCount);


/******************************************************************************
* This function writes a buffer of bytes to the IIC chip.
*
* @param    CoreAddress contains the address of the IIC core.
* @param    ChipAddress contains the address of the chip.
* @param    RegAddress contains the address of the register to write to.
* @param    BufferPtr contains the address of the data to write.
* @param    ByteCount contains the number of bytes in the buffer to be written.
*           Note that this should not exceed the page size as noted by the
*           constant PAGE_SIZE.
*
* @return   The number of bytes written, a value less than that which was
*           specified as an input indicates an error.
*
* @note     None.
*
******************************************************************************/
Xuint8  vsk_iic_write(Xuint32 CoreAddress, Xuint8 ChipAddress, Xuint8 RegAddress,
                  Xuint8 *BufferPtr, Xuint8 ByteCount);



#endif
